{
****************************************************************
*  Name    : 18F24K22                                          *
*  Author  : David John Barker                                 *
*  Notice  : Copyright (c) 2018 Mecanique                      *
*          : All Rights Reserved                               *
*  Date    : 7/29/2018                                         *
****************************************************************
}

module SystemTypes

// system header...
#const _core = $0012                   // processor core
#const _ram_banks = $03                // 3 RAM bank(s) used
#variable _maxaccess = $60             // access ram is 96 bytes
#variable _maxram = $0300              // 768 bytes of user RAM
#variable _maxrom = $004000            // 16 KB of program ROM
#const _eeprom = $0100                 // 256 bytes of EEPROM
#const _eeprom_start = $F00000         // EEPROM start address
#const _nvmcon = $00                   // eeprom uses EECON1
#const _ports = $04                    // 4 available ports
#const _pps = $00                      // PPS module NOT supported
#const _ccp = $02                      // 2 CCP module(s) available
#const _eccp = $03                     // 3 ECCP module(s) available
#const _mssp = $02                     // 2 MSSP module(s) available
#const _usart = $02                    // 2 USART(s) available
#const _adc = $11                      // 17 ADC channels available
#const _adres = $0A                    // 10 bit ADC resolution
#const _comparator = $02               // 2 comparator(s) available
#const _psp = $00                      // Parallel Slave Port (PSP) is NOT supported
#const _can = $00                      // onboard CAN is NOT supported
#const _usb = $00                      // USB is NOT supported
#const _ethernet = $00                 // onboard Ethernet is NOT supported
#const _flash_write = $01              // FLASH has write capability
#const _wdt_type = $02                 // WDTEN watchdog (w/SWON)
#const _ipen = $00                     // IPEN located in RCON

// special function registers...
public system
   ANSELA as byte absolute $0F38,
   ANSELB as byte absolute $0F39,
   ANSELC as byte absolute $0F3A,
   PMD2 as byte absolute $0F3D,
   PMD1 as byte absolute $0F3E,
   PMD0 as byte absolute $0F3F,
   DACCON1 as byte absolute $0F40,
   VREFCON2 as byte absolute $0F40,
   DACCON0 as byte absolute $0F41,
   VREFCON1 as byte absolute $0F41,
   FVRCON as byte absolute $0F42,
   VREFCON0 as byte absolute $0F42,
   CTMUICON as byte absolute $0F43,
   CTMUICONH as byte absolute $0F43,
   CTMUCON1 as byte absolute $0F44,
   CTMUCONL as byte absolute $0F44,
   CTMUCON0 as byte absolute $0F45,
   CTMUCONH as byte absolute $0F45,
   SRCON1 as byte absolute $0F46,
   SRCON0 as byte absolute $0F47,
   CCPTMRS1 as byte absolute $0F48,
   CCPTMRS0 as byte absolute $0F49,
   T6CON as byte absolute $0F4A,
   PR6 as byte absolute $0F4B,
   TMR6 as byte absolute $0F4C,
   T5GCON as byte absolute $0F4D,
   T5CON as byte absolute $0F4E,
   TMR5 as byte absolute $0F4F,
   TMR5L as byte absolute $0F4F,
   TMR5H as byte absolute $0F50,
   T4CON as byte absolute $0F51,
   PR4 as byte absolute $0F52,
   TMR4 as byte absolute $0F53,
   CCP5CON as byte absolute $0F54,
   CCPR5 as byte absolute $0F55,
   CCPR5L as byte absolute $0F55,
   CCPR5H as byte absolute $0F56,
   CCP4CON as byte absolute $0F57,
   CCPR4 as byte absolute $0F58,
   CCPR4L as byte absolute $0F58,
   CCPR4H as byte absolute $0F59,
   PSTR3CON as byte absolute $0F5A,
   CCP3AS as byte absolute $0F5B,
   ECCP3AS as byte absolute $0F5B,
   PWM3CON as byte absolute $0F5C,
   CCP3CON as byte absolute $0F5D,
   CCPR3 as byte absolute $0F5E,
   CCPR3L as byte absolute $0F5E,
   CCPR3H as byte absolute $0F5F,
   SLRCON as byte absolute $0F60,
   WPUB as byte absolute $0F61,
   IOCB as byte absolute $0F62,
   PSTR2CON as byte absolute $0F63,
   CCP2AS as byte absolute $0F64,
   ECCP2AS as byte absolute $0F64,
   PWM2CON as byte absolute $0F65,
   CCP2CON as byte absolute $0F66,
   CCPR2 as byte absolute $0F67,
   CCPR2L as byte absolute $0F67,
   CCPR2H as byte absolute $0F68,
   SSP2CON3 as byte absolute $0F69,
   SSP2MSK as byte absolute $0F6A,
   SSP2CON2 as byte absolute $0F6B,
   SSP2CON1 as byte absolute $0F6C,
   SSP2STAT as byte absolute $0F6D,
   SSP2ADD as byte absolute $0F6E,
   SSP2BUF as byte absolute $0F6F,
   BAUD2CON as byte absolute $0F70,
   BAUDCON2 as byte absolute $0F70,
   RC2STA as byte absolute $0F71,
   RCSTA2 as byte absolute $0F71,
   TX2STA as byte absolute $0F72,
   TXSTA2 as byte absolute $0F72,
   TX2REG as byte absolute $0F73,
   TXREG2 as byte absolute $0F73,
   RC2REG as byte absolute $0F74,
   RCREG2 as byte absolute $0F74,
   SP2BRG as byte absolute $0F75,
   SPBRG2 as byte absolute $0F75,
   SP2BRGH as byte absolute $0F76,
   SPBRGH2 as byte absolute $0F76,
   CM12CON as byte absolute $0F77,
   CM2CON1 as byte absolute $0F77,
   CM2CON as byte absolute $0F78,
   CM2CON0 as byte absolute $0F78,
   CM1CON as byte absolute $0F79,
   CM1CON0 as byte absolute $0F79,
   PIE4 as byte absolute $0F7A,
   PIR4 as byte absolute $0F7B,
   IPR4 as byte absolute $0F7C,
   PIE5 as byte absolute $0F7D,
   PIR5 as byte absolute $0F7E,
   IPR5 as byte absolute $0F7F,
   LATA as byte absolute $0F89,
   LATB as byte absolute $0F8A,
   LATC as byte absolute $0F8B,
   DDRA as byte absolute $0F92,
   TRISA as byte absolute $0F92,
   DDRB as byte absolute $0F93,
   TRISB as byte absolute $0F93,
   DDRC as byte absolute $0F94,
   TRISC as byte absolute $0F94,
   TRISE as byte absolute $0F96,
   OSCTUNE as byte absolute $0F9B,
   HLVDCON as byte absolute $0F9C,
   LVDCON as byte absolute $0F9C,
   PIE1 as byte absolute $0F9D,
   PIR1 as byte absolute $0F9E,
   IPR1 as byte absolute $0F9F,
   PIE2 as byte absolute $0FA0,
   PIR2 as byte absolute $0FA1,
   IPR2 as byte absolute $0FA2,
   PIE3 as byte absolute $0FA3,
   PIR3 as byte absolute $0FA4,
   IPR3 as byte absolute $0FA5,
   EECON1 as byte absolute $0FA6,
   EECON2 as byte absolute $0FA7,
   EEDATA as byte absolute $0FA8,
   EEADR as byte absolute $0FA9,
   RC1STA as byte absolute $0FAB,
   RCSTA as byte absolute $0FAB,
   RCSTA1 as byte absolute $0FAB,
   TX1STA as byte absolute $0FAC,
   TXSTA as byte absolute $0FAC,
   TXSTA1 as byte absolute $0FAC,
   TX1REG as byte absolute $0FAD,
   TXREG as byte absolute $0FAD,
   TXREG1 as byte absolute $0FAD,
   RC1REG as byte absolute $0FAE,
   RCREG as byte absolute $0FAE,
   RCREG1 as byte absolute $0FAE,
   SP1BRG as byte absolute $0FAF,
   SPBRG as byte absolute $0FAF,
   SPBRG1 as byte absolute $0FAF,
   SP1BRGH as byte absolute $0FB0,
   SPBRGH as byte absolute $0FB0,
   SPBRGH1 as byte absolute $0FB0,
   T3CON as byte absolute $0FB1,
   TMR3 as byte absolute $0FB2,
   TMR3L as byte absolute $0FB2,
   TMR3H as byte absolute $0FB3,
   T3GCON as byte absolute $0FB4,
   ECCP1AS as byte absolute $0FB6,
   ECCPAS as byte absolute $0FB6,
   PWM1CON as byte absolute $0FB7,
   PWMCON as byte absolute $0FB7,
   BAUD1CON as byte absolute $0FB8,
   BAUDCON as byte absolute $0FB8,
   BAUDCON1 as byte absolute $0FB8,
   BAUDCTL as byte absolute $0FB8,
   PSTR1CON as byte absolute $0FB9,
   PSTRCON as byte absolute $0FB9,
   T2CON as byte absolute $0FBA,
   PR2 as byte absolute $0FBB,
   TMR2 as byte absolute $0FBC,
   CCP1CON as byte absolute $0FBD,
   CCPR1 as byte absolute $0FBE,
   CCPR1L as byte absolute $0FBE,
   CCPR1H as byte absolute $0FBF,
   ADCON2 as byte absolute $0FC0,
   ADCON1 as byte absolute $0FC1,
   ADCON0 as byte absolute $0FC2,
   ADRES as byte absolute $0FC3,
   ADRESL as byte absolute $0FC3,
   ADRESH as byte absolute $0FC4,
   SSP1CON2 as byte absolute $0FC5,
   SSPCON2 as byte absolute $0FC5,
   SSP1CON1 as byte absolute $0FC6,
   SSPCON1 as byte absolute $0FC6,
   SSP1STAT as byte absolute $0FC7,
   SSPSTAT as byte absolute $0FC7,
   SSP1ADD as byte absolute $0FC8,
   SSPADD as byte absolute $0FC8,
   SSP1BUF as byte absolute $0FC9,
   SSPBUF as byte absolute $0FC9,
   SSP1MSK as byte absolute $0FCA,
   SSPMSK as byte absolute $0FCA,
   SSP1CON3 as byte absolute $0FCB,
   SSPCON3 as byte absolute $0FCB,
   T1GCON as byte absolute $0FCC,
   T1CON as byte absolute $0FCD,
   TMR1 as byte absolute $0FCE,
   TMR1L as byte absolute $0FCE,
   TMR1H as byte absolute $0FCF,
   RCON as byte absolute $0FD0,
   WDTCON as byte absolute $0FD1,
   OSCCON2 as byte absolute $0FD2,
   OSCCON as byte absolute $0FD3,
   T0CON as byte absolute $0FD5,
   TMR0 as byte absolute $0FD6,
   TMR0L as byte absolute $0FD6,
   TMR0H as byte absolute $0FD7,
   STATUS as byte absolute $0FD8,
   FSR2L as byte absolute $0FD9,
   FSR2H as byte absolute $0FDA,
   PLUSW2 as byte absolute $0FDB,
   PREINC2 as byte absolute $0FDC,
   POSTDEC2 as byte absolute $0FDD,
   POSTINC2 as byte absolute $0FDE,
   INDF2 as byte absolute $0FDF,
   BSR as byte absolute $0FE0,
   FSR1L as byte absolute $0FE1,
   FSR1H as byte absolute $0FE2,
   PLUSW1 as byte absolute $0FE3,
   PREINC1 as byte absolute $0FE4,
   POSTDEC1 as byte absolute $0FE5,
   POSTINC1 as byte absolute $0FE6,
   INDF1 as byte absolute $0FE7,
   WREG as byte absolute $0FE8,
   FSR0L as byte absolute $0FE9,
   FSR0H as byte absolute $0FEA,
   PLUSW0 as byte absolute $0FEB,
   PREINC0 as byte absolute $0FEC,
   POSTDEC0 as byte absolute $0FED,
   POSTINC0 as byte absolute $0FEE,
   INDF0 as byte absolute $0FEF,
   INTCON3 as byte absolute $0FF0,
   INTCON2 as byte absolute $0FF1,
   INTCON as byte absolute $0FF2,
   PRODL as byte absolute $0FF3,
   PRODH as byte absolute $0FF4,
   TABLAT as byte absolute $0FF5,
   TBLPTR as byte absolute $0FF6,
   TBLPTRL as byte absolute $0FF6,
   TBLPTRH as byte absolute $0FF7,
   TBLPTRU as byte absolute $0FF8,
   PC as byte absolute $0FF9,
   PCL as byte absolute $0FF9,
   PCLATH as byte absolute $0FFA,
   PCLATU as byte absolute $0FFB,
   STKPTR as byte absolute $0FFC,
   TOS as byte absolute $0FFD,
   TOSL as byte absolute $0FFD,
   TOSH as byte absolute $0FFE,
   TOSU as byte absolute $0FFF

// system ports...
public system port
   PORTA as byte absolute $0F80,
   PORTB as byte absolute $0F81,
   PORTC as byte absolute $0F82,
   PORTE as byte absolute $0F84

// alias...
public dim
   PROD as PRODL.AsWord,
   FSR0 as FSR0L.AsWord,
   FSR1 as FSR1L.AsWord,
   FSR2 as FSR2L.AsWord,
   TABLEPTR as TBLPTRL.AsWord

// configuration fuses...
public config
   FOSC(FOSC) = [LP, XT, HSHP, HSMP, ECHP, ECHPIO6, RC, RCIO6, INTIO67, INTIO7, ECMP, ECMPIO6, ECLP, ECLPIO6],
   PLLCFG(PLLCFG) = [OFF, ON],
   PRICLKEN(PRICLKEN) = [OFF, ON],
   FCMEN(FCMEN) = [OFF, ON],
   IESO(IESO) = [OFF, ON],
   PWRTEN(PWRTEN) = [ON, OFF],
   BOREN(BOREN) = [OFF, ON, NOSLP, SBORDIS],
   BORV(BORV) = [285, 250, 220, 190],
   WDTEN(WDTEN) = [OFF, NOSLP, SWON, ON],
   WDTPS(WDTPS) = [1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768],
   CCP2MX(CCP2MX) = [PORTB3, PORTC1],
   PBADEN(PBADEN) = [OFF, ON],
   CCP3MX(CCP3MX) = [PORTC6, PORTB5],
   HFOFST(HFOFST) = [OFF, ON],
   T3CMX(T3CMX) = [PORTB5, PORTC0],
   P2BMX(P2BMX) = [PORTC0, PORTB5],
   MCLRE(MCLRE) = [INTMCLR, EXTMCLR],
   STVREN(STVREN) = [OFF, ON],
   LVP(LVP) = [OFF, ON],
   XINST(XINST) = [OFF, ON],
   DEBUG(DEBUG) = [ON, OFF],
   CP0(CP0) = [ON, OFF],
   CP1(CP1) = [ON, OFF],
   CPB(CPB) = [ON, OFF],
   CPD(CPD) = [ON, OFF],
   WRT0(WRT0) = [ON, OFF],
   WRT1(WRT1) = [ON, OFF],
   WRTC(WRTC) = [ON, OFF],
   WRTB(WRTB) = [ON, OFF],
   WRTD(WRTD) = [ON, OFF],
   EBTR0(EBTR0) = [ON, OFF],
   EBTR1(EBTR1) = [ON, OFF],
   EBTRB(EBTRB) = [ON, OFF]

// default fuses...
config
   FOSC = HSHP,
   FCMEN = OFF,
   IESO = OFF,
   BOREN = ON,
   WDTEN = OFF,
   WDTPS = 128,
   PBADEN = OFF,
   STVREN = ON,
   LVP = OFF,
   XINST = OFF,
   DEBUG = OFF

end module
