{
****************************************************************
*  Name    : 18F24K50                                          *
*  Author  : David John Barker                                 *
*  Notice  : Copyright (c) 2018 Mecanique                      *
*          : All Rights Reserved                               *
*  Date    : 7/29/2018                                         *
****************************************************************
}

module SystemTypes

// system header...
#const _core = $0012                   // processor core
#const _ram_banks = $08                // 8 RAM bank(s) used
#variable _maxaccess = $60             // access ram is 96 bytes
#variable _maxram = $0800              // 2048 bytes of user RAM
#variable _maxrom = $004000            // 16 KB of program ROM
#const _eeprom = $0100                 // 256 bytes of EEPROM
#const _eeprom_start = $F00000         // EEPROM start address
#const _nvmcon = $00                   // eeprom uses EECON1
#const _ports = $04                    // 4 available ports
#const _pps = $00                      // PPS module NOT supported
#const _ccp = $01                      // 1 CCP module(s) available
#const _eccp = $01                     // 1 ECCP module(s) available
#const _mssp = $01                     // 1 MSSP module(s) available
#const _usart = $01                    // 1 USART(s) available
#const _adc = $05                      // 5 ADC channels available
#const _adres = $0A                    // 10 bit ADC resolution
#const _comparator = $02               // 2 comparator(s) available
#const _psp = $00                      // Parallel Slave Port (PSP) is NOT supported
#const _can = $00                      // onboard CAN is NOT supported
#const _usb = $01                      // onboard USB available
#const _ethernet = $00                 // onboard Ethernet is NOT supported
#const _flash_write = $01              // FLASH has write capability
#const _wdt_type = $02                 // WDTEN watchdog (w/SWON)
#const _ipen = $00                     // IPEN located in RCON

// special function registers...
public system
   SRCON1 as byte absolute $0F57,
   SRCON0 as byte absolute $0F58,
   CCPTMRS as byte absolute $0F59,
   VREGCON as byte absolute $0F5A,
   ANSELA as byte absolute $0F5B,
   ANSELB as byte absolute $0F5C,
   ANSELC as byte absolute $0F5D,
   UCON as byte absolute $0F60,
   USTAT as byte absolute $0F61,
   UCFG as byte absolute $0F62,
   UADDR as byte absolute $0F63,
   UIE as byte absolute $0F64,
   UIR as byte absolute $0F65,
   UEIE as byte absolute $0F66,
   UEIR as byte absolute $0F67,
   UFRM as byte absolute $0F68,
   UFRML as byte absolute $0F68,
   UFRMH as byte absolute $0F69,
   UEP0 as byte absolute $0F6A,
   UEP1 as byte absolute $0F6B,
   UEP2 as byte absolute $0F6C,
   UEP3 as byte absolute $0F6D,
   UEP4 as byte absolute $0F6E,
   UEP5 as byte absolute $0F6F,
   UEP6 as byte absolute $0F70,
   UEP7 as byte absolute $0F71,
   UEP8 as byte absolute $0F72,
   UEP9 as byte absolute $0F73,
   UEP10 as byte absolute $0F74,
   UEP11 as byte absolute $0F75,
   UEP12 as byte absolute $0F76,
   UEP13 as byte absolute $0F77,
   UEP14 as byte absolute $0F78,
   UEP15 as byte absolute $0F79,
   SLRCON as byte absolute $0F7A,
   VREFCON2 as byte absolute $0F7B,
   VREFCON1 as byte absolute $0F7C,
   VREFCON0 as byte absolute $0F7D,
   PMD0 as byte absolute $0F7E,
   PMD1 as byte absolute $0F7F,
   WPUB as byte absolute $0F85,
   IOCB as byte absolute $0F86,
   IOCC as byte absolute $0F87,
   CTMUICON as byte absolute $0F88,
   LATA as byte absolute $0F89,
   LATB as byte absolute $0F8A,
   LATC as byte absolute $0F8B,
   CTMUCON1 as byte absolute $0F8E,
   CTMUCONL as byte absolute $0F8E,
   CTMUCON0 as byte absolute $0F8F,
   CTMUCONH as byte absolute $0F8F,
   CCPR2 as byte absolute $0F90,
   CCPR2L as byte absolute $0F90,
   CCPR2H as byte absolute $0F91,
   DDRA as byte absolute $0F92,
   TRISA as byte absolute $0F92,
   DDRB as byte absolute $0F93,
   TRISB as byte absolute $0F93,
   DDRC as byte absolute $0F94,
   TRISC as byte absolute $0F94,
   CCP2CON as byte absolute $0F97,
   ECCP2CON as byte absolute $0F97,
   CM1CON0 as byte absolute $0F98,
   CM2CON0 as byte absolute $0F99,
   CM2CON1 as byte absolute $0F9A,
   OSCTUNE as byte absolute $0F9B,
   HLVDCON as byte absolute $0F9C,
   PIE1 as byte absolute $0F9D,
   PIR1 as byte absolute $0F9E,
   IPR1 as byte absolute $0F9F,
   PIE2 as byte absolute $0FA0,
   PIR2 as byte absolute $0FA1,
   IPR2 as byte absolute $0FA2,
   PIE3 as byte absolute $0FA3,
   PIR3 as byte absolute $0FA4,
   IPR3 as byte absolute $0FA5,
   EECON1 as byte absolute $0FA6,
   EECON2 as byte absolute $0FA7,
   EEDATA as byte absolute $0FA8,
   EEADR as byte absolute $0FA9,
   RCSTA as byte absolute $0FAB,
   RCSTA1 as byte absolute $0FAB,
   TXSTA as byte absolute $0FAC,
   TXSTA1 as byte absolute $0FAC,
   TXREG as byte absolute $0FAD,
   TXREG1 as byte absolute $0FAD,
   RCREG as byte absolute $0FAE,
   RCREG1 as byte absolute $0FAE,
   SPBRG as byte absolute $0FAF,
   SPBRG1 as byte absolute $0FAF,
   SPBRGH as byte absolute $0FB0,
   SPBRGH1 as byte absolute $0FB0,
   T3CON as byte absolute $0FB1,
   TMR3 as byte absolute $0FB2,
   TMR3L as byte absolute $0FB2,
   TMR3H as byte absolute $0FB3,
   T3GCON as byte absolute $0FB4,
   ACTCON as byte absolute $0FB5,
   STCON as byte absolute $0FB5,
   ECCP1AS as byte absolute $0FB6,
   ECCP1DEL as byte absolute $0FB7,
   PWM1CON as byte absolute $0FB7,
   BAUDCON as byte absolute $0FB8,
   BAUDCON1 as byte absolute $0FB8,
   PSTR1CON as byte absolute $0FB9,
   T2CON as byte absolute $0FBA,
   PR2 as byte absolute $0FBB,
   TMR2 as byte absolute $0FBC,
   CCP1CON as byte absolute $0FBD,
   ECCP1CON as byte absolute $0FBD,
   CCPR1 as byte absolute $0FBE,
   CCPR1L as byte absolute $0FBE,
   CCPR1H as byte absolute $0FBF,
   ADCON2 as byte absolute $0FC0,
   ADCON1 as byte absolute $0FC1,
   ADCON0 as byte absolute $0FC2,
   ADRESL as byte absolute $0FC3,
   ADRESH as byte absolute $0FC4,
   SSP1CON2 as byte absolute $0FC5,
   SSPCON2 as byte absolute $0FC5,
   SSP1CON1 as byte absolute $0FC6,
   SSPCON1 as byte absolute $0FC6,
   SSPCON as byte absolute $0FC6,
   SSP1STAT as byte absolute $0FC7,
   SSPSTAT as byte absolute $0FC7,
   SSP1ADD as byte absolute $0FC8,
   SSPADD as byte absolute $0FC8,
   SSP1BUF as byte absolute $0FC9,
   SSPBUF as byte absolute $0FC9,
   SSP1MSK as byte absolute $0FCA,
   SSPMSK as byte absolute $0FCA,
   SSP1CON3 as byte absolute $0FCB,
   SSPCON3 as byte absolute $0FCB,
   T1GCON as byte absolute $0FCC,
   T1CON as byte absolute $0FCD,
   TMR1 as byte absolute $0FCE,
   TMR1L as byte absolute $0FCE,
   TMR1H as byte absolute $0FCF,
   RCON as byte absolute $0FD0,
   WDTCON as byte absolute $0FD1,
   OSCCON2 as byte absolute $0FD2,
   OSCCON as byte absolute $0FD3,
   T0CON as byte absolute $0FD5,
   TMR0 as byte absolute $0FD6,
   TMR0L as byte absolute $0FD6,
   TMR0H as byte absolute $0FD7,
   STATUS as byte absolute $0FD8,
   FSR2L as byte absolute $0FD9,
   FSR2H as byte absolute $0FDA,
   PLUSW2 as byte absolute $0FDB,
   PREINC2 as byte absolute $0FDC,
   POSTDEC2 as byte absolute $0FDD,
   POSTINC2 as byte absolute $0FDE,
   INDF2 as byte absolute $0FDF,
   BSR as byte absolute $0FE0,
   FSR1L as byte absolute $0FE1,
   FSR1H as byte absolute $0FE2,
   PLUSW1 as byte absolute $0FE3,
   PREINC1 as byte absolute $0FE4,
   POSTDEC1 as byte absolute $0FE5,
   POSTINC1 as byte absolute $0FE6,
   INDF1 as byte absolute $0FE7,
   WREG as byte absolute $0FE8,
   FSR0L as byte absolute $0FE9,
   FSR0H as byte absolute $0FEA,
   PLUSW0 as byte absolute $0FEB,
   PREINC0 as byte absolute $0FEC,
   POSTDEC0 as byte absolute $0FED,
   POSTINC0 as byte absolute $0FEE,
   INDF0 as byte absolute $0FEF,
   INTCON3 as byte absolute $0FF0,
   INTCON2 as byte absolute $0FF1,
   INTCON as byte absolute $0FF2,
   PRODL as byte absolute $0FF3,
   PRODH as byte absolute $0FF4,
   TABLAT as byte absolute $0FF5,
   TBLPTR as byte absolute $0FF6,
   TBLPTRL as byte absolute $0FF6,
   TBLPTRH as byte absolute $0FF7,
   TBLPTRU as byte absolute $0FF8,
   PC as byte absolute $0FF9,
   PCL as byte absolute $0FF9,
   PCLATH as byte absolute $0FFA,
   PCLATU as byte absolute $0FFB,
   STKPTR as byte absolute $0FFC,
   TOS as byte absolute $0FFD,
   TOSL as byte absolute $0FFD,
   TOSH as byte absolute $0FFE,
   TOSU as byte absolute $0FFF

// system ports...
public system port
   PORTA as byte absolute $0F80,
   PORTB as byte absolute $0F81,
   PORTC as byte absolute $0F82,
   PORTE as byte absolute $0F84

// alias...
public dim
   PROD as PRODL.AsWord,
   FSR0 as FSR0L.AsWord,
   FSR1 as FSR1L.AsWord,
   FSR2 as FSR2L.AsWord,
   TABLEPTR as TBLPTRL.AsWord

// configuration fuses...
public config
   PLLSEL(PLLSEL) = [PLL4X, PLL3X],
   CFGPLLEN(CFGPLLEN) = [OFF, ON],
   CPUDIV(CPUDIV) = [NOCLKDIV, CLKDIV2, CLKDIV3, CLKDIV6],
   LS48MHZ(LS48MHZ) = [SYS24X4, SYS48X8],
   FOSC(FOSC) = [LP, XT, HSH, HSM, ECHCLKO, ECHIO, RCCLKO, RCIO, INTOSCIO, INTOSCCLKO, ECMCLKO, ECMIO, ECLCLKO, ECLIO],
   PCLKEN(PCLKEN) = [OFF, ON],
   FCMEN(FCMEN) = [OFF, ON],
   IESO(IESO) = [OFF, ON],
   nPWRTEN(nPWRTEN) = [ON, OFF],
   BOREN(BOREN) = [OFF, ON, NOSLP, SBORDIS],
   BORV(BORV) = [285, 250, 220, 190],
   nLPBOR(nLPBOR) = [ON, OFF],
   WDTEN(WDTEN) = [OFF, NOSLP, SWON, ON],
   WDTPS(WDTPS) = [1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768],
   CCP2MX(CCP2MX) = [RB3, RC1],
   PBADEN(PBADEN) = [OFF, ON],
   T3CMX(T3CMX) = [RB5, RC0],
   SDOMX(SDOMX) = [RC7, RB3],
   MCLRE(MCLRE) = [OFF, ON],
   STVREN(STVREN) = [OFF, ON],
   LVP(LVP) = [OFF, ON],
   ICPRT(ICPRT) = [OFF],
   XINST(XINST) = [OFF, ON],
   DEBUG(DEBUG) = [ON, OFF],
   CP0(CP0) = [ON, OFF],
   CP1(CP1) = [ON, OFF],
   CPB(CPB) = [ON, OFF],
   CPD(CPD) = [ON, OFF],
   WRT0(WRT0) = [ON, OFF],
   WRT1(WRT1) = [ON, OFF],
   WRTC(WRTC) = [ON, OFF],
   WRTB(WRTB) = [ON, OFF],
   WRTD(WRTD) = [ON, OFF],
   EBTR0(EBTR0) = [ON, OFF],
   EBTR1(EBTR1) = [ON, OFF],
   EBTRB(EBTRB) = [ON, OFF]

// default fuses...
config
   FCMEN = OFF,
   IESO = OFF,
   BOREN = ON,
   WDTEN = OFF,
   WDTPS = 128,
   PBADEN = OFF,
   STVREN = ON,
   LVP = OFF,
   ICPRT = OFF,
   XINST = OFF,
   DEBUG = OFF

end module
