{
****************************************************************
*  Name    : 18F24Q10                                          *
*  Author  : David John Barker                                 *
*  Notice  : Copyright (c) 2018 Mecanique                      *
*          : All Rights Reserved                               *
*  Date    : 7/29/2018                                         *
****************************************************************
}

module SystemTypes

// system header...
#const _core = $0012                   // processor core
#const _ram_banks = $04                // 4 RAM bank(s) used
#variable _maxaccess = $60             // access ram is 96 bytes
#variable _maxram = $0400              // 1024 bytes of user RAM
#variable _maxrom = $004000            // 16 KB of program ROM
#const _eeprom = $0100                 // 256 bytes of EEPROM
#const _eeprom_start = $310000         // EEPROM start address
#const _nvmcon = $02                   // eeprom uses NVMCON0/NVMCON1
#const _ports = $04                    // 4 available ports
#const _pps = $01                      // PPS module available
#const _ccp = $02                      // 2 CCP module(s) available
#const _eccp = $00                     // ECCP module NOT supported
#const _mssp = $01                     // 1 MSSP module(s) available
#const _usart = $01                    // 1 USART(s) available
#const _adc = $00                      // 0 ADC channels available
#const _comparator = $02               // 2 comparator(s) available
#const _psp = $00                      // Parallel Slave Port (PSP) is NOT supported
#const _can = $00                      // onboard CAN is NOT supported
#const _usb = $00                      // USB is NOT supported
#const _ethernet = $00                 // onboard Ethernet is NOT supported
#const _flash_write = $01              // FLASH has write capability
#const _wdt_type = $04                 // WDTE watchdog
#const _ipen = $01                     // IPEN located in INTCON

// special function registers...
public system
   PPSLOCK as byte absolute $0E9B,
   INT0PPS as byte absolute $0E9C,
   INT1PPS as byte absolute $0E9D,
   INT2PPS as byte absolute $0E9E,
   T0CKIPPS as byte absolute $0E9F,
   T1CKIPPS as byte absolute $0EA0,
   T1GPPS as byte absolute $0EA1,
   T3CKIPPS as byte absolute $0EA2,
   T3GPPS as byte absolute $0EA3,
   T5CKIPPS as byte absolute $0EA4,
   T5GPPS as byte absolute $0EA5,
   T2INPPS as byte absolute $0EA6,
   T4INPPS as byte absolute $0EA7,
   T6INPPS as byte absolute $0EA8,
   ADACTPPS as byte absolute $0EA9,
   CCP1PPS as byte absolute $0EAA,
   CCP2PPS as byte absolute $0EAB,
   CWG1INPPS as byte absolute $0EAC,
   CWG1PPS as byte absolute $0EAC,
   CWGINPPS as byte absolute $0EAC,
   MDCARLPPS as byte absolute $0EAD,
   MDCARHPPS as byte absolute $0EAE,
   MDSRCPPS as byte absolute $0EAF,
   RX1PPS as byte absolute $0EB0,
   RXDT1PPS as byte absolute $0EB0,
   RXPPS as byte absolute $0EB0,
   CK1PPS as byte absolute $0EB1,
   CKPPS as byte absolute $0EB1,
   TX1PPS as byte absolute $0EB1,
   TXCK1PPS as byte absolute $0EB1,
   TXPPS as byte absolute $0EB1,
   SSP1CLKPPS as byte absolute $0EB2,
   SSPCLKPPS as byte absolute $0EB2,
   SSP1DATPPS as byte absolute $0EB3,
   SSPDATPPS as byte absolute $0EB3,
   SSP1SSPPS as byte absolute $0EB4,
   SSPSSPPS as byte absolute $0EB4,
   IPR0 as byte absolute $0EB5,
   IPR1 as byte absolute $0EB6,
   IPR2 as byte absolute $0EB7,
   IPR3 as byte absolute $0EB8,
   IPR4 as byte absolute $0EB9,
   IPR5 as byte absolute $0EBA,
   IPR6 as byte absolute $0EBB,
   IPR7 as byte absolute $0EBC,
   PIE0 as byte absolute $0EBD,
   PIE1 as byte absolute $0EBE,
   PIE2 as byte absolute $0EBF,
   PIE3 as byte absolute $0EC0,
   PIE4 as byte absolute $0EC1,
   PIE5 as byte absolute $0EC2,
   PIE6 as byte absolute $0EC3,
   PIE7 as byte absolute $0EC4,
   PIR0 as byte absolute $0EC5,
   PIR1 as byte absolute $0EC6,
   PIR2 as byte absolute $0EC7,
   PIR3 as byte absolute $0EC8,
   PIR4 as byte absolute $0EC9,
   PIR5 as byte absolute $0ECA,
   PIR6 as byte absolute $0ECB,
   PIR7 as byte absolute $0ECC,
   WDTCON0 as byte absolute $0ECD,
   WDTCON1 as byte absolute $0ECE,
   WDTPSL as byte absolute $0ECF,
   WDTPSH as byte absolute $0ED0,
   WDTTMR as byte absolute $0ED1,
   CPUDOZE as byte absolute $0ED2,
   OSCCON1 as byte absolute $0ED3,
   OSCCON2 as byte absolute $0ED4,
   OSCCON3 as byte absolute $0ED5,
   OSCSTAT as byte absolute $0ED6,
   OSCSTAT1 as byte absolute $0ED6,
   OSCEN as byte absolute $0ED7,
   OSCTUNE as byte absolute $0ED8,
   OSCFREQ as byte absolute $0ED9,
   OSCFRQ as byte absolute $0ED9,
   VREGCON as byte absolute $0EDA,
   BORCON as byte absolute $0EDB,
   PMD0 as byte absolute $0EDC,
   PMD1 as byte absolute $0EDD,
   PMD2 as byte absolute $0EDE,
   PMD3 as byte absolute $0EDF,
   PMD4 as byte absolute $0EE0,
   PMD5 as byte absolute $0EE1,
   RA0PPS as byte absolute $0EE2,
   RA1PPS as byte absolute $0EE3,
   RA2PPS as byte absolute $0EE4,
   RA3PPS as byte absolute $0EE5,
   RA4PPS as byte absolute $0EE6,
   RA5PPS as byte absolute $0EE7,
   RA6PPS as byte absolute $0EE8,
   RA7PPS as byte absolute $0EE9,
   RB0PPS as byte absolute $0EEA,
   RB1PPS as byte absolute $0EEB,
   RB2PPS as byte absolute $0EEC,
   RB3PPS as byte absolute $0EED,
   RB4PPS as byte absolute $0EEE,
   RB5PPS as byte absolute $0EEF,
   RB6PPS as byte absolute $0EF0,
   RB7PPS as byte absolute $0EF1,
   RC0PPS as byte absolute $0EF2,
   RC1PPS as byte absolute $0EF3,
   RC2PPS as byte absolute $0EF4,
   RC3PPS as byte absolute $0EF5,
   RC4PPS as byte absolute $0EF6,
   RC5PPS as byte absolute $0EF7,
   RC6PPS as byte absolute $0EF8,
   RC7PPS as byte absolute $0EF9,
   IOCAF as byte absolute $0F05,
   IOCAN as byte absolute $0F06,
   IOCAP as byte absolute $0F07,
   INLVLA as byte absolute $0F08,
   SLRCONA as byte absolute $0F09,
   ODCONA as byte absolute $0F0A,
   WPUA as byte absolute $0F0B,
   ANSELA as byte absolute $0F0C,
   IOCBF as byte absolute $0F0D,
   IOCBN as byte absolute $0F0E,
   IOCBP as byte absolute $0F0F,
   INLVLB as byte absolute $0F10,
   SLRCONB as byte absolute $0F11,
   ODCONB as byte absolute $0F12,
   WPUB as byte absolute $0F13,
   ANSELB as byte absolute $0F14,
   IOCCF as byte absolute $0F15,
   IOCCN as byte absolute $0F16,
   IOCCP as byte absolute $0F17,
   INLVLC as byte absolute $0F18,
   SLRCONC as byte absolute $0F19,
   ODCONC as byte absolute $0F1A,
   WPUC as byte absolute $0F1B,
   ANSELC as byte absolute $0F1C,
   IOCEF as byte absolute $0F22,
   IOCEN as byte absolute $0F23,
   IOCEP as byte absolute $0F24,
   INLVLE as byte absolute $0F25,
   WPUE as byte absolute $0F28,
   HLVDCON0 as byte absolute $0F2A,
   HLVDCON1 as byte absolute $0F2B,
   FVRCON as byte absolute $0F2C,
   ZCDCON as byte absolute $0F2D,
   DAC1CON0 as byte absolute $0F2E,
   DAC1CON1 as byte absolute $0F2F,
   CM2CON0 as byte absolute $0F30,
   CM2CON1 as byte absolute $0F31,
   CM2NCH as byte absolute $0F32,
   CM2PCH as byte absolute $0F33,
   CM1CON0 as byte absolute $0F34,
   CM1CON1 as byte absolute $0F35,
   CM1NCH as byte absolute $0F36,
   CM1PCH as byte absolute $0F37,
   CMOUT as byte absolute $0F38,
   CLKRCON as byte absolute $0F39,
   CLKRCLK as byte absolute $0F3A,
   CWG1CLK as byte absolute $0F3B,
   CWG1CLKCON as byte absolute $0F3B,
   CWG1ISM as byte absolute $0F3C,
   CWG1DBR as byte absolute $0F3D,
   CWG1DBF as byte absolute $0F3E,
   CWG1CON0 as byte absolute $0F3F,
   CWG1CON1 as byte absolute $0F40,
   CWG1AS0 as byte absolute $0F41,
   CWG1AS1 as byte absolute $0F42,
   CWG1STR as byte absolute $0F43,
   SCANLADR as byte absolute $0F44,
   SCANLADRL as byte absolute $0F44,
   SCANLADRH as byte absolute $0F45,
   SCANLADRU as byte absolute $0F46,
   SCANHADR as byte absolute $0F47,
   SCANHADRL as byte absolute $0F47,
   SCANHADRH as byte absolute $0F48,
   SCANHADRU as byte absolute $0F49,
   SCANCON0 as byte absolute $0F4A,
   SCANTRIG as byte absolute $0F4B,
   MDCON0 as byte absolute $0F4C,
   MDCON1 as byte absolute $0F4D,
   MDSRC as byte absolute $0F4E,
   MDCARL as byte absolute $0F4F,
   MDCARH as byte absolute $0F50,
   ADACT as byte absolute $0F51,
   ADCLK as byte absolute $0F52,
   ADREF as byte absolute $0F53,
   ADCON1 as byte absolute $0F54,
   ADCON2 as byte absolute $0F55,
   ADCON3 as byte absolute $0F56,
   ADACQ as byte absolute $0F57,
   ADCAP as byte absolute $0F58,
   ADPRE as byte absolute $0F59,
   ADPCH as byte absolute $0F5A,
   ADCON0 as byte absolute $0F5B,
   ADPREV as byte absolute $0F5C,
   ADPREVL as byte absolute $0F5C,
   ADPREVH as byte absolute $0F5D,
   ADRES as byte absolute $0F5E,
   ADRESL as byte absolute $0F5E,
   ADRESH as byte absolute $0F5F,
   ADSTAT as byte absolute $0F60,
   ADRPT as byte absolute $0F61,
   ADCNT as byte absolute $0F62,
   ADSTPT as byte absolute $0F63,
   ADSTPTL as byte absolute $0F63,
   ADSTPTH as byte absolute $0F64,
   ADLTH as byte absolute $0F65,
   ADLTHL as byte absolute $0F65,
   ADLTHH as byte absolute $0F66,
   ADUTH as byte absolute $0F67,
   ADUTHL as byte absolute $0F67,
   ADUTHH as byte absolute $0F68,
   ADERR as byte absolute $0F69,
   ADERRL as byte absolute $0F69,
   ADERRH as byte absolute $0F6A,
   ADACC as byte absolute $0F6B,
   ADACCL as byte absolute $0F6B,
   ADACCH as byte absolute $0F6C,
   ADFLTR as byte absolute $0F6D,
   ADFLTRL as byte absolute $0F6D,
   ADFLTRH as byte absolute $0F6E,
   CRCDATA as byte absolute $0F6F,
   CRCDATL as byte absolute $0F6F,
   CRCDATH as byte absolute $0F70,
   CRCACC as byte absolute $0F71,
   CRCACCL as byte absolute $0F71,
   CRCACCH as byte absolute $0F72,
   CRCSHFT as byte absolute $0F73,
   CRCSHIFTL as byte absolute $0F73,
   CRCSHIFTH as byte absolute $0F74,
   CRCXOR as byte absolute $0F75,
   CRCXORL as byte absolute $0F75,
   CRCXORH as byte absolute $0F76,
   CRCCON0 as byte absolute $0F77,
   CRCCON1 as byte absolute $0F78,
   NVMADR as byte absolute $0F79,
   NVMADRL as byte absolute $0F79,
   NVMADRH as byte absolute $0F7A,
   NVMADRU as byte absolute $0F7B,
   NVMDAT as byte absolute $0F7C,
   NVMDATL as byte absolute $0F7C,
   NVMDATH as byte absolute $0F7D,
   NVMCON0 as byte absolute $0F7F,
   NVMCON1 as byte absolute $0F80,
   NVMCON2 as byte absolute $0F81,
   LATA as byte absolute $0F82,
   LATB as byte absolute $0F83,
   LATC as byte absolute $0F84,
   DDRA as byte absolute $0F87,
   TRISA as byte absolute $0F87,
   DDRB as byte absolute $0F88,
   TRISB as byte absolute $0F88,
   DDRC as byte absolute $0F89,
   TRISC as byte absolute $0F89,
   SSP1BUF as byte absolute $0F91,
   SSPBUF as byte absolute $0F91,
   SSP1ADD as byte absolute $0F92,
   SSPADD as byte absolute $0F92,
   SSP1MSK as byte absolute $0F93,
   SSPMSK as byte absolute $0F93,
   SSP1STAT as byte absolute $0F94,
   SSPSTAT as byte absolute $0F94,
   SSP1CON1 as byte absolute $0F95,
   SSPCON1 as byte absolute $0F95,
   SSP1CON2 as byte absolute $0F96,
   SSPCON2 as byte absolute $0F96,
   SSP1CON3 as byte absolute $0F97,
   SSPCON3 as byte absolute $0F97,
   RC1REG as byte absolute $0F98,
   RCREG as byte absolute $0F98,
   RCREG1 as byte absolute $0F98,
   TX1REG as byte absolute $0F99,
   TXREG as byte absolute $0F99,
   TXREG1 as byte absolute $0F99,
   SP1BRG as byte absolute $0F9A,
   SP1BRGL as byte absolute $0F9A,
   SPBRG as byte absolute $0F9A,
   SPBRG1 as byte absolute $0F9A,
   SPBRGL as byte absolute $0F9A,
   SP1BRGH as byte absolute $0F9B,
   SPBRGH as byte absolute $0F9B,
   SPBRGH1 as byte absolute $0F9B,
   RC1STA as byte absolute $0F9C,
   RCSTA as byte absolute $0F9C,
   RCSTA1 as byte absolute $0F9C,
   TX1STA as byte absolute $0F9D,
   TXSTA as byte absolute $0F9D,
   TXSTA1 as byte absolute $0F9D,
   BAUD1CON as byte absolute $0F9E,
   BAUDCON as byte absolute $0F9E,
   BAUDCON1 as byte absolute $0F9E,
   BAUDCTL as byte absolute $0F9E,
   BAUDCTL1 as byte absolute $0F9E,
   PWM4DCL as byte absolute $0F9F,
   PWM4DCH as byte absolute $0FA0,
   PWM4CON as byte absolute $0FA1,
   PWM3DCL as byte absolute $0FA2,
   PWM3DCH as byte absolute $0FA3,
   PWM3CON as byte absolute $0FA4,
   CCPR2 as byte absolute $0FA5,
   CCPR2L as byte absolute $0FA5,
   CCPR2H as byte absolute $0FA6,
   CCP2CON as byte absolute $0FA7,
   CCP2CAP as byte absolute $0FA8,
   CCPR1 as byte absolute $0FA9,
   CCPR1L as byte absolute $0FA9,
   CCPR1H as byte absolute $0FAA,
   CCP1CON as byte absolute $0FAB,
   CCP1CAP as byte absolute $0FAC,
   CCPTMRS as byte absolute $0FAD,
   T6TMR as byte absolute $0FAE,
   TMR6 as byte absolute $0FAE,
   PR6 as byte absolute $0FAF,
   T6PR as byte absolute $0FAF,
   T6CON as byte absolute $0FB0,
   T6HLT as byte absolute $0FB1,
   T6CLK as byte absolute $0FB2,
   T6CLKCON as byte absolute $0FB2,
   T6RST as byte absolute $0FB3,
   T4TMR as byte absolute $0FB4,
   TMR4 as byte absolute $0FB4,
   PR4 as byte absolute $0FB5,
   T4PR as byte absolute $0FB5,
   T4CON as byte absolute $0FB6,
   T4HLT as byte absolute $0FB7,
   T4CLK as byte absolute $0FB8,
   T4CLKCON as byte absolute $0FB8,
   T4RST as byte absolute $0FB9,
   T2TMR as byte absolute $0FBA,
   TMR2 as byte absolute $0FBA,
   PR2 as byte absolute $0FBB,
   T2PR as byte absolute $0FBB,
   T2CON as byte absolute $0FBC,
   T2HLT as byte absolute $0FBD,
   T2CLK as byte absolute $0FBE,
   T2CLKCON as byte absolute $0FBE,
   T2RST as byte absolute $0FBF,
   TMR5 as byte absolute $0FC0,
   TMR5L as byte absolute $0FC0,
   TMR5H as byte absolute $0FC1,
   T5CON as byte absolute $0FC2,
   PR5 as byte absolute $0FC3,
   T5GCON as byte absolute $0FC3,
   T5GATE as byte absolute $0FC4,
   TMR5GATE as byte absolute $0FC4,
   T5CLK as byte absolute $0FC5,
   TMR5CLK as byte absolute $0FC5,
   TMR3 as byte absolute $0FC6,
   TMR3L as byte absolute $0FC6,
   TMR3H as byte absolute $0FC7,
   T3CON as byte absolute $0FC8,
   PR3 as byte absolute $0FC9,
   T3GCON as byte absolute $0FC9,
   T3GATE as byte absolute $0FCA,
   TMR3GATE as byte absolute $0FCA,
   T3CLK as byte absolute $0FCB,
   TMR3CLK as byte absolute $0FCB,
   TMR1 as byte absolute $0FCC,
   TMR1L as byte absolute $0FCC,
   TMR1H as byte absolute $0FCD,
   T1CON as byte absolute $0FCE,
   PR1 as byte absolute $0FCF,
   T1GCON as byte absolute $0FCF,
   T1GATE as byte absolute $0FD0,
   TMR1GATE as byte absolute $0FD0,
   T1CLK as byte absolute $0FD1,
   TMR1CLK as byte absolute $0FD1,
   TMR0 as byte absolute $0FD2,
   TMR0L as byte absolute $0FD2,
   PR0 as byte absolute $0FD3,
   TMR0H as byte absolute $0FD3,
   T0CON0 as byte absolute $0FD4,
   T0CON1 as byte absolute $0FD5,
   PCON1 as byte absolute $0FD6,
   PCON0 as byte absolute $0FD7,
   STATUS as byte absolute $0FD8,
   FSR2L as byte absolute $0FD9,
   FSR2H as byte absolute $0FDA,
   PLUSW2 as byte absolute $0FDB,
   PREINC2 as byte absolute $0FDC,
   POSTDEC2 as byte absolute $0FDD,
   POSTINC2 as byte absolute $0FDE,
   INDF2 as byte absolute $0FDF,
   BSR as byte absolute $0FE0,
   FSR1L as byte absolute $0FE1,
   FSR1H as byte absolute $0FE2,
   PLUSW1 as byte absolute $0FE3,
   PREINC1 as byte absolute $0FE4,
   POSTDEC1 as byte absolute $0FE5,
   POSTINC1 as byte absolute $0FE6,
   INDF1 as byte absolute $0FE7,
   WREG as byte absolute $0FE8,
   FSR0L as byte absolute $0FE9,
   FSR0H as byte absolute $0FEA,
   PLUSW0 as byte absolute $0FEB,
   PREINC0 as byte absolute $0FEC,
   POSTDEC0 as byte absolute $0FED,
   POSTINC0 as byte absolute $0FEE,
   INDF0 as byte absolute $0FEF,
   INTCON as byte absolute $0FF2,
   PRODL as byte absolute $0FF3,
   PRODH as byte absolute $0FF4,
   TABLAT as byte absolute $0FF5,
   TBLPTR as byte absolute $0FF6,
   TBLPTRL as byte absolute $0FF6,
   TBLPTRH as byte absolute $0FF7,
   TBLPTRU as byte absolute $0FF8,
   PC as byte absolute $0FF9,
   PCL as byte absolute $0FF9,
   PCLATH as byte absolute $0FFA,
   PCLATU as byte absolute $0FFB,
   STKPTR as byte absolute $0FFC,
   TOS as byte absolute $0FFD,
   TOSL as byte absolute $0FFD,
   TOSH as byte absolute $0FFE,
   TOSU as byte absolute $0FFF

// system ports...
public system port
   PORTA as byte absolute $0F8C,
   PORTB as byte absolute $0F8D,
   PORTC as byte absolute $0F8E,
   PORTE as byte absolute $0F90

// alias...
public dim
   PROD as PRODL.AsWord,
   FSR0 as FSR0L.AsWord,
   FSR1 as FSR1L.AsWord,
   FSR2 as FSR2L.AsWord,
   TABLEPTR as TBLPTRL.AsWord

// configuration fuses...
public config
   FEXTOSC(FEXTOSC) = [LP, XT, HS, OFF, ECL, ECM, ECH],
   RSTOSC(RSTOSC) = [HFINTOSC_64MHZ, HFINTOSC_16MHZ, EXTOSC_4PLL, RESERVED_2, SOSC, LFINTOSC, HFINTOSC_1MHZ, EXTOSC],
   CLKOUTEN(CLKOUTEN) = [ON, OFF],
   CSWEN(CSWEN) = [OFF, ON],
   FCMEN(FCMEN) = [OFF, ON],
   MCLRE(MCLRE) = [INTMCLR, EXTMCLR],
   PWRTE(PWRTE) = [ON, OFF],
   LPBOREN(LPBOREN) = [ON, OFF],
   BOREN(BOREN) = [OFF, ON, NOSLP, SBORDIS],
   BORV(BORV) = [VBOR_285, VBOR_270, VBOR_245, VBOR_2P45],
   ZCD(ZCD) = [ON, OFF],
   PPS1WAY(PPS1WAY) = [OFF, ON],
   STVREN(STVREN) = [OFF, ON],
   DEBUG(DEBUG) = [ON, OFF],
   XINST(XINST) = [ON, OFF],
   WDTCPS(WDTCPS) = [WDTCPS_0, WDTCPS_1, WDTCPS_2, WDTCPS_3, WDTCPS_4, WDTCPS_5, WDTCPS_6, WDTCPS_7, WDTCPS_8, WDTCPS_9, WDTCPS_10, WDTCPS_11, WDTCPS_12, WDTCPS_13, WDTCPS_14, WDTCPS_15, WDTCPS_16, WDTCPS_17, WDTCPS_18, WDTCPS_19, WDTCPS_20, WDTCPS_21, WDTCPS_22, WDTCPS_23, WDTCPS_24, WDTCPS_25, WDTCPS_26, WDTCPS_27, WDTCPS_28, WDTCPS_29, WDTCPS_30, WDTCPS_31],
   WDTE(WDTE) = [OFF, SWDTEN, NSLEEP, ON],
   WDTCWS(WDTCWS) = [WDTCWS_0, WDTCWS_1, WDTCWS_2, WDTCWS_3, WDTCWS_4, WDTCWS_5, WDTCWS_6, WDTCWS_7],
   WDTCCS(WDTCCS) = [LFINTOSC, HFINTOSC, SC],
   WRT0(WRT0) = [ON, OFF],
   WRT1(WRT1) = [ON, OFF],
   WRTC(WRTC) = [ON, OFF],
   WRTB(WRTB) = [ON, OFF],
   WRTD(WRTD) = [ON, OFF],
   SCANE(SCANE) = [OFF, ON],
   LVP(LVP) = [OFF, ON],
   CP(CP) = [ON, OFF],
   CPD(CPD) = [ON, OFF],
   EBTR0(EBTR0) = [ON, OFF],
   EBTR1(EBTR1) = [ON, OFF],
   EBTRB(EBTRB) = [ON, OFF]

// default fuses...
config
   FEXTOSC = HS,
   RSTOSC = HFINTOSC_1MHZ,
   FCMEN = OFF,
   BOREN = ON,
   STVREN = ON,
   DEBUG = OFF,
   XINST = OFF,
   WDTE = OFF,
   LVP = OFF

//
// SF interrupt startup code fix for IPEN (K40, Q10, and others)
//
// the compiler assumes that the IPEN interrupt priority bit is in RCON.bits(7)
// some devices have moved this bit to INTCON.bits(5), and have no RCON register
// to satisfy the compiler we define an RCON register symbol at the same
// address as the PRODL register... that way code will compile and it does
// no harm when the compiler sets/clears the bit.
// when using interrupts SF will set/clear RCON.bits(7) automatically at the
// start of MAIN, so right after that we check the bit (which is now PRODL.7)
// and set the real IPEN bit in INTCON to match.
// this code will execute even if not using interrupts, but should have no
// detrimental effects
//
#if (_ipen = 1)
// the real IPEN bit
public dim IPEN as INTCON.bits(5)
// define dummy RCON as PRODL to satisfy compiler
private system RCON as byte absolute $0FF3
asm
  RCON EQU 0x0FF3
end asm

// replicate the RCON.bits(7) setting into the real IPEN
IPEN = 0
if (RCON.bits(7) = 1) then
    IPEN = 1
endif
#endif      // (_ipen = 1)

end module
