{
****************************************************************
*  Name    : 18F25K80                                          *
*  Author  : David John Barker                                 *
*  Notice  : Copyright (c) 2018 Mecanique                      *
*          : All Rights Reserved                               *
*  Date    : 7/29/2018                                         *
****************************************************************
}

module SystemTypes

// system header...
#const _core = $0012                   // processor core
#const _ram_banks = $0F                // 15 RAM bank(s) used
#variable _maxaccess = $60             // access ram is 96 bytes
#variable _maxram = $0E41              // 3649 bytes of user RAM
#variable _maxrom = $008000            // 32 KB of program ROM
#const _eeprom = $0400                 // 1024 bytes of EEPROM
#const _eeprom_start = $F00000         // EEPROM start address
#const _nvmcon = $00                   // eeprom uses EECON1
#const _ports = $04                    // 4 available ports
#const _pps = $00                      // PPS module NOT supported
#const _ccp = $04                      // 4 CCP module(s) available
#const _eccp = $01                     // 1 ECCP module(s) available
#const _mssp = $01                     // 1 MSSP module(s) available
#const _usart = $02                    // 2 USART(s) available
#const _adc = $05                      // 5 ADC channels available
#const _adres = $0A                    // 10 bit ADC resolution
#const _comparator = $00               // comparator module NOT supported
#const _psp = $00                      // Parallel Slave Port (PSP) is NOT supported
#const _can = $01                      // onboard CAN available
#const _usb = $00                      // USB is NOT supported
#const _ethernet = $00                 // onboard Ethernet is NOT supported
#const _flash_write = $01              // FLASH has write capability
#const _wdt_type = $03                 // WDTEN watchdog (w/SWDTDIS)
#const _ipen = $00                     // IPEN located in RCON

// special function registers...
public system
   RXERRCNT as byte absolute $0E41,
   TXERRCNT as byte absolute $0E42,
   BRGCON1 as byte absolute $0E43,
   BRGCON2 as byte absolute $0E44,
   BRGCON3 as byte absolute $0E45,
   RXFCON0 as byte absolute $0E46,
   RXFCON1 as byte absolute $0E47,
   RXF6SIDH as byte absolute $0E48,
   RXF6SIDL as byte absolute $0E49,
   RXF6EIDH as byte absolute $0E4A,
   RXF6EIDL as byte absolute $0E4B,
   RXF7SIDH as byte absolute $0E4C,
   RXF7SIDL as byte absolute $0E4D,
   RXF7EIDH as byte absolute $0E4E,
   RXF7EIDL as byte absolute $0E4F,
   RXF8SIDH as byte absolute $0E50,
   RXF8SIDL as byte absolute $0E51,
   RXF8EIDH as byte absolute $0E52,
   RXF8EIDL as byte absolute $0E53,
   RXF9SIDH as byte absolute $0E54,
   RXF9SIDL as byte absolute $0E55,
   RXF9EIDH as byte absolute $0E56,
   RXF9EIDL as byte absolute $0E57,
   RXF10SIDH as byte absolute $0E58,
   RXF10SIDL as byte absolute $0E59,
   RXF10EIDH as byte absolute $0E5A,
   RXF10EIDL as byte absolute $0E5B,
   RXF11SIDH as byte absolute $0E5C,
   RXF11SIDL as byte absolute $0E5D,
   RXF11EIDH as byte absolute $0E5E,
   RXF11EIDL as byte absolute $0E5F,
   RXF12SIDH as byte absolute $0E60,
   RXF12SIDL as byte absolute $0E61,
   RXF12EIDH as byte absolute $0E62,
   RXF12EIDL as byte absolute $0E63,
   RXF13SIDH as byte absolute $0E64,
   RXF13SIDL as byte absolute $0E65,
   RXF13EIDH as byte absolute $0E66,
   RXF13EIDL as byte absolute $0E67,
   RXF14SIDH as byte absolute $0E68,
   RXF14SIDL as byte absolute $0E69,
   RXF14EIDH as byte absolute $0E6A,
   RXF14EIDL as byte absolute $0E6B,
   RXF15SIDH as byte absolute $0E6C,
   RXF15SIDL as byte absolute $0E6D,
   RXF15EIDH as byte absolute $0E6E,
   RXF15EIDL as byte absolute $0E6F,
   SDFLC as byte absolute $0E70,
   RXFBCON0 as byte absolute $0E71,
   RXFBCON1 as byte absolute $0E72,
   RXFBCON2 as byte absolute $0E73,
   RXFBCON3 as byte absolute $0E74,
   RXFBCON4 as byte absolute $0E75,
   RXFBCON5 as byte absolute $0E76,
   RXFBCON6 as byte absolute $0E77,
   RXFBCON7 as byte absolute $0E78,
   MSEL0 as byte absolute $0E79,
   MSEL1 as byte absolute $0E7A,
   MSEL2 as byte absolute $0E7B,
   MSEL3 as byte absolute $0E7C,
   BSEL0 as byte absolute $0E7D,
   BIE0 as byte absolute $0E7E,
   TXBIE as byte absolute $0E7F,
   B0CON as byte absolute $0E80,
   B0SIDH as byte absolute $0E81,
   B0SIDL as byte absolute $0E82,
   B0EIDH as byte absolute $0E83,
   B0EIDL as byte absolute $0E84,
   B0DLC as byte absolute $0E85,
   B0D0 as byte absolute $0E86,
   B0D1 as byte absolute $0E87,
   B0D2 as byte absolute $0E88,
   B0D3 as byte absolute $0E89,
   B0D4 as byte absolute $0E8A,
   B0D5 as byte absolute $0E8B,
   B0D6 as byte absolute $0E8C,
   B0D7 as byte absolute $0E8D,
   CANSTAT_RO9 as byte absolute $0E8E,
   CANCON_RO9 as byte absolute $0E8F,
   B1CON as byte absolute $0E90,
   B1SIDH as byte absolute $0E91,
   B1SIDL as byte absolute $0E92,
   B1EIDH as byte absolute $0E93,
   B1EIDL as byte absolute $0E94,
   B1DLC as byte absolute $0E95,
   B1D0 as byte absolute $0E96,
   B1D1 as byte absolute $0E97,
   B1D2 as byte absolute $0E98,
   B1D3 as byte absolute $0E99,
   B1D4 as byte absolute $0E9A,
   B1D5 as byte absolute $0E9B,
   B1D6 as byte absolute $0E9C,
   B1D7 as byte absolute $0E9D,
   CANSTAT_RO8 as byte absolute $0E9E,
   CANCON_RO8 as byte absolute $0E9F,
   B2CON as byte absolute $0EA0,
   B2SIDH as byte absolute $0EA1,
   B2SIDL as byte absolute $0EA2,
   B2EIDH as byte absolute $0EA3,
   B2EIDL as byte absolute $0EA4,
   B2DLC as byte absolute $0EA5,
   B2D0 as byte absolute $0EA6,
   B2D1 as byte absolute $0EA7,
   B2D2 as byte absolute $0EA8,
   B2D3 as byte absolute $0EA9,
   B2D4 as byte absolute $0EAA,
   B2D5 as byte absolute $0EAB,
   B2D6 as byte absolute $0EAC,
   B2D7 as byte absolute $0EAD,
   CANSTAT_RO7 as byte absolute $0EAE,
   CANCON_RO7 as byte absolute $0EAF,
   B3CON as byte absolute $0EB0,
   B3SIDH as byte absolute $0EB1,
   B3SIDL as byte absolute $0EB2,
   B3EIDH as byte absolute $0EB3,
   B3EIDL as byte absolute $0EB4,
   B3DLC as byte absolute $0EB5,
   B3D0 as byte absolute $0EB6,
   B3D1 as byte absolute $0EB7,
   B3D2 as byte absolute $0EB8,
   B3D3 as byte absolute $0EB9,
   B3D4 as byte absolute $0EBA,
   B3D5 as byte absolute $0EBB,
   B3D6 as byte absolute $0EBC,
   B3D7 as byte absolute $0EBD,
   CANSTAT_RO6 as byte absolute $0EBE,
   CANCON_RO6 as byte absolute $0EBF,
   B4CON as byte absolute $0EC0,
   B4SIDH as byte absolute $0EC1,
   B4SIDL as byte absolute $0EC2,
   B4EIDH as byte absolute $0EC3,
   B4EIDL as byte absolute $0EC4,
   B4DLC as byte absolute $0EC5,
   B4D0 as byte absolute $0EC6,
   B4D1 as byte absolute $0EC7,
   B4D2 as byte absolute $0EC8,
   B4D3 as byte absolute $0EC9,
   B4D4 as byte absolute $0ECA,
   B4D5 as byte absolute $0ECB,
   B4D6 as byte absolute $0ECC,
   B4D7 as byte absolute $0ECD,
   CANSTAT_RO5 as byte absolute $0ECE,
   CANCON_RO5 as byte absolute $0ECF,
   B5CON as byte absolute $0ED0,
   B5SIDH as byte absolute $0ED1,
   B5SIDL as byte absolute $0ED2,
   B5EIDH as byte absolute $0ED3,
   B5EIDL as byte absolute $0ED4,
   B5DLC as byte absolute $0ED5,
   B5D0 as byte absolute $0ED6,
   B5D1 as byte absolute $0ED7,
   B5D2 as byte absolute $0ED8,
   B5D3 as byte absolute $0ED9,
   B5D4 as byte absolute $0EDA,
   B5D5 as byte absolute $0EDB,
   B5D6 as byte absolute $0EDC,
   B5D7 as byte absolute $0EDD,
   CANSTAT_RO4 as byte absolute $0EDE,
   CANCON_RO4 as byte absolute $0EDF,
   RXF0SIDH as byte absolute $0EE0,
   RXF0SIDL as byte absolute $0EE1,
   RXF0EIDH as byte absolute $0EE2,
   RXF0EIDL as byte absolute $0EE3,
   RXF1SIDH as byte absolute $0EE4,
   RXF1SIDL as byte absolute $0EE5,
   RXF1EIDH as byte absolute $0EE6,
   RXF1EIDL as byte absolute $0EE7,
   RXF2SIDH as byte absolute $0EE8,
   RXF2SIDL as byte absolute $0EE9,
   RXF2EIDH as byte absolute $0EEA,
   RXF2EIDL as byte absolute $0EEB,
   RXF3SIDH as byte absolute $0EEC,
   RXF3SIDL as byte absolute $0EED,
   RXF3EIDH as byte absolute $0EEE,
   RXF3EIDL as byte absolute $0EEF,
   RXF4SIDH as byte absolute $0EF0,
   RXF4SIDL as byte absolute $0EF1,
   RXF4EIDH as byte absolute $0EF2,
   RXF4EIDL as byte absolute $0EF3,
   RXF5SIDH as byte absolute $0EF4,
   RXF5SIDL as byte absolute $0EF5,
   RXF5EIDH as byte absolute $0EF6,
   RXF5EIDL as byte absolute $0EF7,
   RXM0SIDH as byte absolute $0EF8,
   RXM0SIDL as byte absolute $0EF9,
   RXM0EIDH as byte absolute $0EFA,
   RXM0EIDL as byte absolute $0EFB,
   RXM1SIDH as byte absolute $0EFC,
   RXM1SIDL as byte absolute $0EFD,
   RXM1EIDH as byte absolute $0EFE,
   RXM1EIDL as byte absolute $0EFF,
   TXB2CON as byte absolute $0F00,
   TXB2SIDH as byte absolute $0F01,
   TXB2SIDL as byte absolute $0F02,
   TXB2EIDH as byte absolute $0F03,
   TXB2EIDL as byte absolute $0F04,
   TXB2DLC as byte absolute $0F05,
   TXB2D0 as byte absolute $0F06,
   TXB2D1 as byte absolute $0F07,
   TXB2D2 as byte absolute $0F08,
   TXB2D3 as byte absolute $0F09,
   TXB2D4 as byte absolute $0F0A,
   TXB2D5 as byte absolute $0F0B,
   TXB2D6 as byte absolute $0F0C,
   TXB2D7 as byte absolute $0F0D,
   CANSTAT_RO3 as byte absolute $0F0E,
   CANCON_RO3 as byte absolute $0F0F,
   TXB1CON as byte absolute $0F10,
   TXB1SIDH as byte absolute $0F11,
   TXB1SIDL as byte absolute $0F12,
   TXB1EIDH as byte absolute $0F13,
   TXB1EIDL as byte absolute $0F14,
   TXB1DLC as byte absolute $0F15,
   TXB1D0 as byte absolute $0F16,
   TXB1D1 as byte absolute $0F17,
   TXB1D2 as byte absolute $0F18,
   TXB1D3 as byte absolute $0F19,
   TXB1D4 as byte absolute $0F1A,
   TXB1D5 as byte absolute $0F1B,
   TXB1D6 as byte absolute $0F1C,
   TXB1D7 as byte absolute $0F1D,
   CANSTAT_RO2 as byte absolute $0F1E,
   CANCON_RO2 as byte absolute $0F1F,
   TXB0CON as byte absolute $0F20,
   TXB0SIDH as byte absolute $0F21,
   TXB0SIDL as byte absolute $0F22,
   TXB0EIDH as byte absolute $0F23,
   TXB0EIDL as byte absolute $0F24,
   TXB0DLC as byte absolute $0F25,
   TXB0D0 as byte absolute $0F26,
   TXB0D1 as byte absolute $0F27,
   TXB0D2 as byte absolute $0F28,
   TXB0D3 as byte absolute $0F29,
   TXB0D4 as byte absolute $0F2A,
   TXB0D5 as byte absolute $0F2B,
   TXB0D6 as byte absolute $0F2C,
   TXB0D7 as byte absolute $0F2D,
   CANSTAT_RO1 as byte absolute $0F2E,
   CANCON_RO1 as byte absolute $0F2F,
   RXB1CON as byte absolute $0F30,
   RXB1SIDH as byte absolute $0F31,
   RXB1SIDL as byte absolute $0F32,
   RXB1EIDH as byte absolute $0F33,
   RXB1EIDL as byte absolute $0F34,
   RXB1DLC as byte absolute $0F35,
   RXB1D0 as byte absolute $0F36,
   RXB1D1 as byte absolute $0F37,
   RXB1D2 as byte absolute $0F38,
   RXB1D3 as byte absolute $0F39,
   RXB1D4 as byte absolute $0F3A,
   RXB1D5 as byte absolute $0F3B,
   RXB1D6 as byte absolute $0F3C,
   RXB1D7 as byte absolute $0F3D,
   CANSTAT_RO0 as byte absolute $0F3E,
   CANCON_RO0 as byte absolute $0F3F,
   CCP5CON as byte absolute $0F47,
   CCPR5 as byte absolute $0F48,
   CCPR5L as byte absolute $0F48,
   CCPR5H as byte absolute $0F49,
   CCP4CON as byte absolute $0F4A,
   CCPR4 as byte absolute $0F4B,
   CCPR4L as byte absolute $0F4B,
   CCPR4H as byte absolute $0F4C,
   CCP3CON as byte absolute $0F4D,
   CCPR3 as byte absolute $0F4E,
   CCPR3L as byte absolute $0F4E,
   CCPR3H as byte absolute $0F4F,
   CCP2CON as byte absolute $0F50,
   ECCP2CON as byte absolute $0F50,
   CCPR2 as byte absolute $0F51,
   CCPR2L as byte absolute $0F51,
   CCPR2H as byte absolute $0F52,
   CTMUICON as byte absolute $0F53,
   CTMUCONL as byte absolute $0F54,
   CTMUCONH as byte absolute $0F55,
   PADCFG1 as byte absolute $0F56,
   PMD2 as byte absolute $0F57,
   PMD1 as byte absolute $0F58,
   PMD0 as byte absolute $0F59,
   IOCB as byte absolute $0F5A,
   WPUB as byte absolute $0F5B,
   ANCON1 as byte absolute $0F5C,
   ANCON0 as byte absolute $0F5D,
   CM2CON as byte absolute $0F5E,
   CM2CON1 as byte absolute $0F5E,
   CM1CON as byte absolute $0F5F,
   CM1CON1 as byte absolute $0F5F,
   RXB0CON as byte absolute $0F60,
   RXB0SIDH as byte absolute $0F61,
   RXB0SIDL as byte absolute $0F62,
   RXB0EIDH as byte absolute $0F63,
   RXB0EIDL as byte absolute $0F64,
   RXB0DLC as byte absolute $0F65,
   RXB0D0 as byte absolute $0F66,
   RXB0D1 as byte absolute $0F67,
   RXB0D2 as byte absolute $0F68,
   RXB0D3 as byte absolute $0F69,
   RXB0D4 as byte absolute $0F6A,
   RXB0D5 as byte absolute $0F6B,
   RXB0D6 as byte absolute $0F6C,
   RXB0D7 as byte absolute $0F6D,
   CANSTAT as byte absolute $0F6E,
   CANCON as byte absolute $0F6F,
   CIOCON as byte absolute $0F70,
   COMSTAT as byte absolute $0F71,
   ECANCON as byte absolute $0F72,
   EEDATA as byte absolute $0F73,
   EEADR as byte absolute $0F74,
   EEADRH as byte absolute $0F75,
   PIE5 as byte absolute $0F76,
   PIR5 as byte absolute $0F77,
   IPR5 as byte absolute $0F78,
   TXREG2 as byte absolute $0F79,
   RCREG2 as byte absolute $0F7A,
   SPBRG2 as byte absolute $0F7B,
   SPBRGH2 as byte absolute $0F7C,
   SPBRGH1 as byte absolute $0F7D,
   EECON2 as byte absolute $0F7E,
   EECON1 as byte absolute $0F7F,
   TMR4 as byte absolute $0F87,
   T4CON as byte absolute $0F88,
   LATA as byte absolute $0F89,
   LATB as byte absolute $0F8A,
   LATC as byte absolute $0F8B,
   SLRCON as byte absolute $0F90,
   ODCON as byte absolute $0F91,
   TRISA as byte absolute $0F92,
   TRISB as byte absolute $0F93,
   TRISC as byte absolute $0F94,
   CCPTMRS as byte absolute $0F99,
   REFOCON as byte absolute $0F9A,
   OSCTUNE as byte absolute $0F9B,
   PSTR1CON as byte absolute $0F9C,
   PIE1 as byte absolute $0F9D,
   PIR1 as byte absolute $0F9E,
   IPR1 as byte absolute $0F9F,
   PIE2 as byte absolute $0FA0,
   PIR2 as byte absolute $0FA1,
   IPR2 as byte absolute $0FA2,
   PIE3 as byte absolute $0FA3,
   PIR3 as byte absolute $0FA4,
   IPR3 as byte absolute $0FA5,
   RCSTA2 as byte absolute $0FA6,
   BAUDCON1 as byte absolute $0FA7,
   HLVDCON as byte absolute $0FA8,
   PR4 as byte absolute $0FA9,
   T1GCON as byte absolute $0FAA,
   RCSTA as byte absolute $0FAB,
   RCSTA1 as byte absolute $0FAB,
   TXSTA as byte absolute $0FAC,
   TXSTA1 as byte absolute $0FAC,
   TXREG as byte absolute $0FAD,
   TXREG1 as byte absolute $0FAD,
   RCREG as byte absolute $0FAE,
   RCREG1 as byte absolute $0FAE,
   SPBRG as byte absolute $0FAF,
   SPBRG1 as byte absolute $0FAF,
   T3GCON as byte absolute $0FB0,
   T3CON as byte absolute $0FB1,
   TMR3 as byte absolute $0FB2,
   TMR3L as byte absolute $0FB2,
   TMR3H as byte absolute $0FB3,
   CMSTAT as byte absolute $0FB4,
   CMSTATUS as byte absolute $0FB4,
   CVRCON as byte absolute $0FB5,
   PIE4 as byte absolute $0FB6,
   PIR4 as byte absolute $0FB7,
   IPR4 as byte absolute $0FB8,
   BAUDCON2 as byte absolute $0FB9,
   TXSTA2 as byte absolute $0FBA,
   CCP1CON as byte absolute $0FBB,
   ECCP1CON as byte absolute $0FBB,
   CCPR1 as byte absolute $0FBC,
   CCPR1L as byte absolute $0FBC,
   CCPR1H as byte absolute $0FBD,
   ECCP1DEL as byte absolute $0FBE,
   PWM1CON as byte absolute $0FBE,
   ECCP1AS as byte absolute $0FBF,
   ADCON2 as byte absolute $0FC0,
   ADCON1 as byte absolute $0FC1,
   ADCON0 as byte absolute $0FC2,
   ADRES as byte absolute $0FC3,
   ADRESL as byte absolute $0FC3,
   ADRESH as byte absolute $0FC4,
   SSPCON2 as byte absolute $0FC5,
   SSPCON1 as byte absolute $0FC6,
   SSPSTAT as byte absolute $0FC7,
   SSPADD as byte absolute $0FC8,
   SSPBUF as byte absolute $0FC9,
   T2CON as byte absolute $0FCA,
   PR2 as byte absolute $0FCB,
   TMR2 as byte absolute $0FCC,
   T1CON as byte absolute $0FCD,
   TMR1 as byte absolute $0FCE,
   TMR1L as byte absolute $0FCE,
   TMR1H as byte absolute $0FCF,
   RCON as byte absolute $0FD0,
   WDTCON as byte absolute $0FD1,
   OSCCON2 as byte absolute $0FD2,
   OSCCON as byte absolute $0FD3,
   T0CON as byte absolute $0FD5,
   TMR0 as byte absolute $0FD6,
   TMR0L as byte absolute $0FD6,
   TMR0H as byte absolute $0FD7,
   STATUS as byte absolute $0FD8,
   FSR2L as byte absolute $0FD9,
   FSR2H as byte absolute $0FDA,
   PLUSW2 as byte absolute $0FDB,
   PREINC2 as byte absolute $0FDC,
   POSTDEC2 as byte absolute $0FDD,
   POSTINC2 as byte absolute $0FDE,
   INDF2 as byte absolute $0FDF,
   BSR as byte absolute $0FE0,
   FSR1L as byte absolute $0FE1,
   FSR1H as byte absolute $0FE2,
   PLUSW1 as byte absolute $0FE3,
   PREINC1 as byte absolute $0FE4,
   POSTDEC1 as byte absolute $0FE5,
   POSTINC1 as byte absolute $0FE6,
   INDF1 as byte absolute $0FE7,
   WREG as byte absolute $0FE8,
   FSR0L as byte absolute $0FE9,
   FSR0H as byte absolute $0FEA,
   PLUSW0 as byte absolute $0FEB,
   PREINC0 as byte absolute $0FEC,
   POSTDEC0 as byte absolute $0FED,
   POSTINC0 as byte absolute $0FEE,
   INDF0 as byte absolute $0FEF,
   INTCON3 as byte absolute $0FF0,
   INTCON2 as byte absolute $0FF1,
   INTCON as byte absolute $0FF2,
   INTCON1 as byte absolute $0FF2,
   PRODL as byte absolute $0FF3,
   PRODH as byte absolute $0FF4,
   TABLAT as byte absolute $0FF5,
   TBLPTR as byte absolute $0FF6,
   TBLPTRL as byte absolute $0FF6,
   TBLPTRH as byte absolute $0FF7,
   TBLPTRU as byte absolute $0FF8,
   PC as byte absolute $0FF9,
   PCL as byte absolute $0FF9,
   PCLATH as byte absolute $0FFA,
   PCLATU as byte absolute $0FFB,
   STKPTR as byte absolute $0FFC,
   TOS as byte absolute $0FFD,
   TOSL as byte absolute $0FFD,
   TOSH as byte absolute $0FFE,
   TOSU as byte absolute $0FFF

// system ports...
public system port
   PORTA as byte absolute $0F80,
   PORTB as byte absolute $0F81,
   PORTC as byte absolute $0F82,
   PORTE as byte absolute $0F84

// alias...
public dim
   PROD as PRODL.AsWord,
   FSR0 as FSR0L.AsWord,
   FSR1 as FSR1L.AsWord,
   FSR2 as FSR2L.AsWord,
   TABLEPTR as TBLPTRL.AsWord

// configuration fuses...
public config
   RETEN(RETEN) = [ON, OFF],
   INTOSCSEL(INTOSCSEL) = [LOW, HIGH],
   SOSCSEL(SOSCSEL) = [LOW, DIG, HIGH],
   XINST(XINST) = [OFF, ON],
   FOSC(FOSC) = [LP, XT, HS2, HS1, EC3IO, EC3, RC, RCIO, INTIO2, INTIO1, EC2IO, EC2, EC1IO, EC1],
   PLLCFG(PLLCFG) = [OFF, ON],
   FCMEN(FCMEN) = [OFF, ON],
   IESO(IESO) = [OFF, ON],
   PWRTEN(PWRTEN) = [ON, OFF],
   BOREN(BOREN) = [OFF, ON, NOSLP, SBORDIS],
   BORV(BORV) = [0, 1, 2, 3],
   BORPWR(BORPWR) = [LOW, MEDIUM, HIGH, ZPBORMV],
   WDTEN(WDTEN) = [OFF, NOSLP, ON, SWDTDIS],
   WDTPS(WDTPS) = [1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, 65536, 131072, 262144, 524288, 1048576],
   CANMX(CANMX) = [PORTC, PORTB],
   MSSPMSK(MSSPMSK) = [MSK5, MSK7],
   MCLRE(MCLRE) = [OFF, ON],
   STVREN(STVREN) = [OFF, ON],
   BBSIZ(BBSIZ) = [BB1K, BB2K],
   CP0(CP0) = [ON, OFF],
   CP1(CP1) = [ON, OFF],
   CP2(CP2) = [ON, OFF],
   CP3(CP3) = [ON, OFF],
   CPB(CPB) = [ON, OFF],
   CPD(CPD) = [ON, OFF],
   WRT0(WRT0) = [ON, OFF],
   WRT1(WRT1) = [ON, OFF],
   WRT2(WRT2) = [ON, OFF],
   WRT3(WRT3) = [ON, OFF],
   WRTC(WRTC) = [ON, OFF],
   WRTB(WRTB) = [ON, OFF],
   WRTD(WRTD) = [ON, OFF],
   EBTR0(EBTR0) = [ON, OFF],
   EBTR1(EBTR1) = [ON, OFF],
   EBTR2(EBTR2) = [ON, OFF],
   EBTR3(EBTR3) = [ON, OFF],
   EBTRB(EBTRB) = [ON, OFF]

// default fuses...
config
   XINST = OFF,
   FCMEN = OFF,
   IESO = OFF,
   BOREN = ON,
   WDTEN = OFF,
   WDTPS = 128,
   STVREN = ON

end module
