{
****************************************************************
*  Name    : 18F66K40                                          *
*  Author  : David John Barker                                 *
*  Notice  : Copyright (c) 2018 Mecanique                      *
*          : All Rights Reserved                               *
*  Date    : 7/29/2018                                         *
****************************************************************
}

module SystemTypes

// system header...
#const _core = $0012                   // processor core
#const _ram_banks = $0E                // 14 RAM bank(s) used
#variable _maxaccess = $60             // access ram is 96 bytes
#variable _maxram = $0DEA              // 3562 bytes of user RAM
#variable _maxrom = $010000            // 64 KB of program ROM
#const _eeprom = $0400                 // 1024 bytes of EEPROM
#const _eeprom_start = $310000         // EEPROM start address
#const _nvmcon = $01                   // eeprom uses NVMCON1
#const _ports = $08                    // 8 available ports
#const _pps = $01                      // PPS module available
#const _ccp = $05                      // 5 CCP module(s) available
#const _eccp = $00                     // ECCP module NOT supported
#const _mssp = $02                     // 2 MSSP module(s) available
#const _usart = $02                    // 2 USART(s) available
#const _adc = $00                      // 0 ADC channels available
#const _comparator = $03               // 3 comparator(s) available
#const _psp = $00                      // Parallel Slave Port (PSP) is NOT supported
#const _can = $00                      // onboard CAN is NOT supported
#const _usb = $00                      // USB is NOT supported
#const _ethernet = $00                 // onboard Ethernet is NOT supported
#const _flash_write = $01              // FLASH has write capability
#const _wdt_type = $04                 // WDTE watchdog
#const _ipen = $01                     // IPEN located in INTCON

// special function registers...
public system
   PPSLOCK as byte absolute $0DEF,
   INT0PPS as byte absolute $0DF0,
   INT1PPS as byte absolute $0DF1,
   INT2PPS as byte absolute $0DF2,
   INT3PPS as byte absolute $0DF3,
   T0CKIPPS as byte absolute $0DF4,
   T1CKIPPS as byte absolute $0DF5,
   T1GPPS as byte absolute $0DF6,
   T3CKIPPS as byte absolute $0DF7,
   T3GPPS as byte absolute $0DF8,
   T5CKIPPS as byte absolute $0DF9,
   T5GPPS as byte absolute $0DFA,
   T7CKIPPS as byte absolute $0DFB,
   T7GPPS as byte absolute $0DFC,
   T2INPPS as byte absolute $0DFD,
   T4INPPS as byte absolute $0DFE,
   T6INPPS as byte absolute $0DFF,
   T8INPPS as byte absolute $0E00,
   ADACTPPS as byte absolute $0E01,
   CCP1PPS as byte absolute $0E02,
   CCP2PPS as byte absolute $0E03,
   CCP3PPS as byte absolute $0E04,
   CCP4PPS as byte absolute $0E05,
   CCP5PPS as byte absolute $0E06,
   SMT1WINPPS as byte absolute $0E07,
   SMT1SIGPPS as byte absolute $0E08,
   SMT2WINPPS as byte absolute $0E09,
   SMT2SIGPPS as byte absolute $0E0A,
   CWG1PPS as byte absolute $0E0B,
   CWGINPPS as byte absolute $0E0B,
   MDCARLPPS as byte absolute $0E0C,
   MDCARHPPS as byte absolute $0E0D,
   MDSRCPPS as byte absolute $0E0E,
   RX1PPS as byte absolute $0E0F,
   RXPPS as byte absolute $0E0F,
   CK1PPS as byte absolute $0E10,
   CKPPS as byte absolute $0E10,
   TX1PPS as byte absolute $0E10,
   TXPPS as byte absolute $0E10,
   RX2PPS as byte absolute $0E11,
   CK2PPS as byte absolute $0E12,
   TX2PPS as byte absolute $0E12,
   RX3PPS as byte absolute $0E13,
   CK3PPS as byte absolute $0E14,
   TX3PPS as byte absolute $0E14,
   RX4PPS as byte absolute $0E15,
   CK4PPS as byte absolute $0E16,
   TX4PPS as byte absolute $0E16,
   RX5PPS as byte absolute $0E17,
   CK5PPS as byte absolute $0E18,
   TX5PPS as byte absolute $0E18,
   SSP1CLKPPS as byte absolute $0E19,
   SSPCLKPPS as byte absolute $0E19,
   SSP1DATPPS as byte absolute $0E1A,
   SSPDATPPS as byte absolute $0E1A,
   SSP1SSPPS as byte absolute $0E1B,
   SSPSSPPS as byte absolute $0E1B,
   SSP2CLKPPS as byte absolute $0E1C,
   SSP2DATPPS as byte absolute $0E1D,
   SSP2SSPPS as byte absolute $0E1E,
   IPR0 as byte absolute $0E1F,
   IPR1 as byte absolute $0E20,
   IPR2 as byte absolute $0E21,
   IPR3 as byte absolute $0E22,
   IPR4 as byte absolute $0E23,
   IPR5 as byte absolute $0E24,
   IPR6 as byte absolute $0E25,
   IPR7 as byte absolute $0E26,
   IPR8 as byte absolute $0E27,
   IPR9 as byte absolute $0E28,
   PIE0 as byte absolute $0E29,
   PIE1 as byte absolute $0E2A,
   PIE2 as byte absolute $0E2B,
   PIE3 as byte absolute $0E2C,
   PIE4 as byte absolute $0E2D,
   PIE5 as byte absolute $0E2E,
   PIE6 as byte absolute $0E2F,
   PIE7 as byte absolute $0E30,
   PIE8 as byte absolute $0E31,
   PIE9 as byte absolute $0E32,
   PIR0 as byte absolute $0E33,
   PIR1 as byte absolute $0E34,
   PIR2 as byte absolute $0E35,
   PIR3 as byte absolute $0E36,
   PIR4 as byte absolute $0E37,
   PIR5 as byte absolute $0E38,
   PIR6 as byte absolute $0E39,
   PIR7 as byte absolute $0E3A,
   PIR8 as byte absolute $0E3B,
   PIR9 as byte absolute $0E3C,
   WDTCON0 as byte absolute $0E3D,
   WDTCON1 as byte absolute $0E3E,
   WDTL as byte absolute $0E3F,
   WDTPSL as byte absolute $0E3F,
   WDTH as byte absolute $0E40,
   WDTPSH as byte absolute $0E40,
   WDTPSU as byte absolute $0E41,
   WDTTMR as byte absolute $0E41,
   WDTU as byte absolute $0E41,
   CPUDOZE as byte absolute $0E42,
   OSCCON1 as byte absolute $0E43,
   OSCCON2 as byte absolute $0E44,
   OSCCON3 as byte absolute $0E45,
   OSCSTAT as byte absolute $0E46,
   OSCSTAT1 as byte absolute $0E46,
   OSCEN as byte absolute $0E47,
   OSCTUNE as byte absolute $0E48,
   OSCFRQ as byte absolute $0E49,
   VREGCON as byte absolute $0E4A,
   BORCON as byte absolute $0E4B,
   PMD0 as byte absolute $0E4C,
   PMD1 as byte absolute $0E4D,
   PMD2 as byte absolute $0E4E,
   PMD3 as byte absolute $0E4F,
   PMD4 as byte absolute $0E50,
   PMD5 as byte absolute $0E51,
   RA0PPS as byte absolute $0E52,
   RA1PPS as byte absolute $0E53,
   RA2PPS as byte absolute $0E54,
   RA3PPS as byte absolute $0E55,
   RA4PPS as byte absolute $0E56,
   RA5PPS as byte absolute $0E57,
   RA6PPS as byte absolute $0E58,
   RA7PPS as byte absolute $0E59,
   RB0PPS as byte absolute $0E5A,
   RB1PPS as byte absolute $0E5B,
   RB2PPS as byte absolute $0E5C,
   RB3PPS as byte absolute $0E5D,
   RB4PPS as byte absolute $0E5E,
   RB5PPS as byte absolute $0E5F,
   RB6PPS as byte absolute $0E60,
   RB7PPS as byte absolute $0E61,
   RC0PPS as byte absolute $0E62,
   RC1PPS as byte absolute $0E63,
   RC2PPS as byte absolute $0E64,
   RC3PPS as byte absolute $0E65,
   RC4PPS as byte absolute $0E66,
   RC5PPS as byte absolute $0E67,
   RC6PPS as byte absolute $0E68,
   RC7PPS as byte absolute $0E69,
   RD0PPS as byte absolute $0E6A,
   RD1PPS as byte absolute $0E6B,
   RD2PPS as byte absolute $0E6C,
   RD3PPS as byte absolute $0E6D,
   RD4PPS as byte absolute $0E6E,
   RD5PPS as byte absolute $0E6F,
   RD6PPS as byte absolute $0E70,
   RD7PPS as byte absolute $0E71,
   RE0PPS as byte absolute $0E72,
   RE1PPS as byte absolute $0E73,
   RE2PPS as byte absolute $0E74,
   RE3PPS as byte absolute $0E75,
   RE4PPS as byte absolute $0E76,
   RE5PPS as byte absolute $0E77,
   RE6PPS as byte absolute $0E78,
   RE7PPS as byte absolute $0E79,
   RF0PPS as byte absolute $0E7A,
   RF1PPS as byte absolute $0E7B,
   RF2PPS as byte absolute $0E7C,
   RF3PPS as byte absolute $0E7D,
   RF4PPS as byte absolute $0E7E,
   RF5PPS as byte absolute $0E7F,
   RF6PPS as byte absolute $0E80,
   RF7PPS as byte absolute $0E81,
   RG0PPS as byte absolute $0E82,
   RG1PPS as byte absolute $0E83,
   RG2PPS as byte absolute $0E84,
   RG3PPS as byte absolute $0E85,
   RG4PPS as byte absolute $0E86,
   RG6PPS as byte absolute $0E88,
   RG7PPS as byte absolute $0E89,
   RH0PPS as byte absolute $0E8A,
   RH1PPS as byte absolute $0E8B,
   RH2PPS as byte absolute $0E8C,
   RH3PPS as byte absolute $0E8D,
   INLVLA as byte absolute $0E8E,
   SLRCONA as byte absolute $0E8F,
   ODCONA as byte absolute $0E90,
   WPUA as byte absolute $0E91,
   ANSELA as byte absolute $0E92,
   IOCBF as byte absolute $0E93,
   IOCBN as byte absolute $0E94,
   IOCBP as byte absolute $0E95,
   INLVLB as byte absolute $0E96,
   SLRCONB as byte absolute $0E97,
   ODCONB as byte absolute $0E98,
   WPUB as byte absolute $0E99,
   ANSELB as byte absolute $0E9A,
   IOCCF as byte absolute $0E9B,
   IOCCN as byte absolute $0E9C,
   IOCCP as byte absolute $0E9D,
   INLVLC as byte absolute $0E9E,
   SLRCONC as byte absolute $0E9F,
   ODCONC as byte absolute $0EA0,
   WPUC as byte absolute $0EA1,
   INLVLD as byte absolute $0EA3,
   SLRCOND as byte absolute $0EA4,
   ODCOND as byte absolute $0EA5,
   WPUD as byte absolute $0EA6,
   ANSELD as byte absolute $0EA7,
   IOCEF as byte absolute $0EA8,
   IOCEN as byte absolute $0EA9,
   IOCEP as byte absolute $0EAA,
   INLVLE as byte absolute $0EAB,
   SLRCONE as byte absolute $0EAC,
   ODCONE as byte absolute $0EAD,
   WPUE as byte absolute $0EAE,
   ANSELE as byte absolute $0EAF,
   INLVLF as byte absolute $0EB0,
   SLRCONF as byte absolute $0EB1,
   ODCONF as byte absolute $0EB2,
   WPUF as byte absolute $0EB3,
   ANSELF as byte absolute $0EB4,
   IOCGF as byte absolute $0EB5,
   IOCGN as byte absolute $0EB6,
   IOCGP as byte absolute $0EB7,
   INLVLG as byte absolute $0EB8,
   SLRCONG as byte absolute $0EB9,
   ODCONG as byte absolute $0EBA,
   WPUG as byte absolute $0EBB,
   ANSELG as byte absolute $0EBC,
   INLVLH as byte absolute $0EBD,
   SLRCONH as byte absolute $0EBE,
   ODCONH as byte absolute $0EBF,
   WPUH as byte absolute $0EC0,
   HLVDCON0 as byte absolute $0EC2,
   HLVDCON1 as byte absolute $0EC3,
   FVRCON as byte absolute $0EC4,
   ZCDCON as byte absolute $0EC5,
   DAC1CON0 as byte absolute $0EC6,
   DAC1CON1 as byte absolute $0EC7,
   CM3CON0 as byte absolute $0EC8,
   CM3CON1 as byte absolute $0EC9,
   CM3NCH as byte absolute $0ECA,
   CM3PCH as byte absolute $0ECB,
   CM2CON0 as byte absolute $0ECC,
   CM2CON1 as byte absolute $0ECD,
   CM2NCH as byte absolute $0ECE,
   CM2PCH as byte absolute $0ECF,
   CM1CON0 as byte absolute $0ED0,
   CM1CON1 as byte absolute $0ED1,
   CM1NCH as byte absolute $0ED2,
   CM1PCH as byte absolute $0ED3,
   CMOUT as byte absolute $0ED4,
   SSP2BUF as byte absolute $0ED5,
   SSP2ADD as byte absolute $0ED6,
   SSP2MSK as byte absolute $0ED7,
   SSP2STAT as byte absolute $0ED8,
   SSP2CON1 as byte absolute $0ED9,
   SSP2CON2 as byte absolute $0EDA,
   SSP2CON3 as byte absolute $0EDB,
   RC5REG as byte absolute $0EDC,
   RCREG5 as byte absolute $0EDC,
   TX5REG as byte absolute $0EDD,
   TXREG5 as byte absolute $0EDD,
   SP5BRG as byte absolute $0EDE,
   SP5BRGL as byte absolute $0EDE,
   SPBRG5 as byte absolute $0EDE,
   SP5BRGH as byte absolute $0EDF,
   SPBRGH5 as byte absolute $0EDF,
   RC5STA as byte absolute $0EE0,
   RCSTA5 as byte absolute $0EE0,
   TX5STA as byte absolute $0EE1,
   TXSTA5 as byte absolute $0EE1,
   BAUD5CON as byte absolute $0EE2,
   BAUDCON5 as byte absolute $0EE2,
   BAUDCTL5 as byte absolute $0EE2,
   RC4REG as byte absolute $0EE3,
   RCREG4 as byte absolute $0EE3,
   TX4REG as byte absolute $0EE4,
   TXREG4 as byte absolute $0EE4,
   SP4BRG as byte absolute $0EE5,
   SP4BRGL as byte absolute $0EE5,
   SPBRG4 as byte absolute $0EE5,
   SP4BRGH as byte absolute $0EE6,
   SPBRGH4 as byte absolute $0EE6,
   RC4STA as byte absolute $0EE7,
   RCSTA4 as byte absolute $0EE7,
   TX4STA as byte absolute $0EE8,
   TXSTA4 as byte absolute $0EE8,
   BAUD4CON as byte absolute $0EE9,
   BAUDCON4 as byte absolute $0EE9,
   BAUDCTL4 as byte absolute $0EE9,
   RC3REG as byte absolute $0EEA,
   RCREG3 as byte absolute $0EEA,
   TX3REG as byte absolute $0EEB,
   TXREG3 as byte absolute $0EEB,
   SP3BRG as byte absolute $0EEC,
   SP3BRGL as byte absolute $0EEC,
   SPBRG3 as byte absolute $0EEC,
   SP3BRGH as byte absolute $0EED,
   SPBRGH3 as byte absolute $0EED,
   RC3STA as byte absolute $0EEE,
   RCSTA3 as byte absolute $0EEE,
   TX3STA as byte absolute $0EEF,
   TXSTA3 as byte absolute $0EEF,
   BAUD3CON as byte absolute $0EF0,
   BAUDCON3 as byte absolute $0EF0,
   BAUDCTL3 as byte absolute $0EF0,
   RC2REG as byte absolute $0EF1,
   RCREG2 as byte absolute $0EF1,
   TX2REG as byte absolute $0EF2,
   TXREG2 as byte absolute $0EF2,
   SP2BRG as byte absolute $0EF3,
   SP2BRGL as byte absolute $0EF3,
   SPBRG2 as byte absolute $0EF3,
   SP2BRGH as byte absolute $0EF4,
   SPBRGH2 as byte absolute $0EF4,
   RC2STA as byte absolute $0EF5,
   RCSTA2 as byte absolute $0EF5,
   TX2STA as byte absolute $0EF6,
   TXSTA2 as byte absolute $0EF6,
   BAUD2CON as byte absolute $0EF7,
   BAUDCON2 as byte absolute $0EF7,
   BAUDCTL2 as byte absolute $0EF7,
   SMT2TMR as byte absolute $0EF8,
   SMT2TMRL as byte absolute $0EF8,
   SMT2TMRH as byte absolute $0EF9,
   SMT2TMRU as byte absolute $0EFA,
   SMT2CPR as byte absolute $0EFB,
   SMT2CPRL as byte absolute $0EFB,
   SMT2CPRH as byte absolute $0EFC,
   SMT2CPRU as byte absolute $0EFD,
   SMT2CPW as byte absolute $0EFE,
   SMT2CPWL as byte absolute $0EFE,
   SMT2CPWH as byte absolute $0EFF,
   SMT2CPWU as byte absolute $0F00,
   SMT2PR as byte absolute $0F01,
   SMT2PRL as byte absolute $0F01,
   SMT2PRH as byte absolute $0F02,
   SMT2PRU as byte absolute $0F03,
   SMT2CON0 as byte absolute $0F04,
   SMT2CON1 as byte absolute $0F05,
   SMT2STAT as byte absolute $0F06,
   SMT2CLK as byte absolute $0F07,
   SMT2SIG as byte absolute $0F08,
   SMT2WIN as byte absolute $0F09,
   SMT1TMR as byte absolute $0F0A,
   SMT1TMRL as byte absolute $0F0A,
   SMT1TMRH as byte absolute $0F0B,
   SMT1TMRU as byte absolute $0F0C,
   SMT1CPR as byte absolute $0F0D,
   SMT1CPRL as byte absolute $0F0D,
   SMT1CPRH as byte absolute $0F0E,
   SMT1CPRU as byte absolute $0F0F,
   SMT1CPW as byte absolute $0F10,
   SMT1CPWL as byte absolute $0F10,
   SMT1CPWH as byte absolute $0F11,
   SMT1CPWU as byte absolute $0F12,
   SMT1PR as byte absolute $0F13,
   SMT1PRL as byte absolute $0F13,
   SMT1PRH as byte absolute $0F14,
   SMT1PRU as byte absolute $0F15,
   SMT1CON0 as byte absolute $0F16,
   SMT1CON1 as byte absolute $0F17,
   SMT1STAT as byte absolute $0F18,
   SMT1CLK as byte absolute $0F19,
   SMT1SIG as byte absolute $0F1A,
   SMT1WIN as byte absolute $0F1B,
   CCPR5 as byte absolute $0F1C,
   CCPR5L as byte absolute $0F1C,
   CCPR5H as byte absolute $0F1D,
   CCP5CON as byte absolute $0F1E,
   CCP5CAP as byte absolute $0F1F,
   CCPR4 as byte absolute $0F20,
   CCPR4L as byte absolute $0F20,
   CCPR4H as byte absolute $0F21,
   CCP4CON as byte absolute $0F22,
   CCP4CAP as byte absolute $0F23,
   CCPR3 as byte absolute $0F24,
   CCPR3L as byte absolute $0F24,
   CCPR3H as byte absolute $0F25,
   CCP3CON as byte absolute $0F26,
   CCP3CAP as byte absolute $0F27,
   T8TMR as byte absolute $0F28,
   TMR8 as byte absolute $0F28,
   PR8 as byte absolute $0F29,
   T8PR as byte absolute $0F29,
   T8CON as byte absolute $0F2A,
   T8HLT as byte absolute $0F2B,
   T8CLKCON as byte absolute $0F2C,
   T8RST as byte absolute $0F2D,
   TMR7 as byte absolute $0F2E,
   TMR7L as byte absolute $0F2E,
   TMR7H as byte absolute $0F2F,
   T7CON as byte absolute $0F30,
   PR7 as byte absolute $0F31,
   T7GCON as byte absolute $0F31,
   T7GATE as byte absolute $0F32,
   TMR7GATE as byte absolute $0F32,
   T7CLK as byte absolute $0F33,
   TMR7CLK as byte absolute $0F33,
   CLKRCON as byte absolute $0F34,
   CLKRCLK as byte absolute $0F35,
   CWG1CLK as byte absolute $0F36,
   CWG1CLKCON as byte absolute $0F36,
   CWG1ISM as byte absolute $0F37,
   CWG1DBR as byte absolute $0F38,
   CWG1DBF as byte absolute $0F39,
   CWG1CON0 as byte absolute $0F3A,
   CWG1CON1 as byte absolute $0F3B,
   CWG1AS0 as byte absolute $0F3C,
   CWG1AS1 as byte absolute $0F3D,
   CWG1STR as byte absolute $0F3E,
   CRCDATA as byte absolute $0F3F,
   CRCDATL as byte absolute $0F3F,
   CRCDATH as byte absolute $0F40,
   CRCACC as byte absolute $0F41,
   CRCACCL as byte absolute $0F41,
   CRCACCH as byte absolute $0F42,
   CRCSHFT as byte absolute $0F43,
   CRCSHIFTL as byte absolute $0F43,
   CRCSHIFTH as byte absolute $0F44,
   CRCXOR as byte absolute $0F45,
   CRCXORL as byte absolute $0F45,
   CRCXORH as byte absolute $0F46,
   CRCCON0 as byte absolute $0F47,
   CRCCON1 as byte absolute $0F48,
   SCANLADR as byte absolute $0F49,
   SCANLADRL as byte absolute $0F49,
   SCANLADRH as byte absolute $0F4A,
   SCANLADRU as byte absolute $0F4B,
   SCANHADR as byte absolute $0F4C,
   SCANHADRL as byte absolute $0F4C,
   SCANHADRH as byte absolute $0F4D,
   SCANHADRU as byte absolute $0F4E,
   SCANCON0 as byte absolute $0F4F,
   SCANTRIG as byte absolute $0F50,
   MDCON0 as byte absolute $0F51,
   MDCON1 as byte absolute $0F52,
   MDSRC as byte absolute $0F53,
   MDCARL as byte absolute $0F54,
   MDCARH as byte absolute $0F55,
   ADACT as byte absolute $0F56,
   ADCLK as byte absolute $0F57,
   ADREF as byte absolute $0F58,
   ADCON1 as byte absolute $0F59,
   ADCON2 as byte absolute $0F5A,
   ADCON3 as byte absolute $0F5B,
   ADACQ as byte absolute $0F5C,
   ADCAP as byte absolute $0F5D,
   ADPRE as byte absolute $0F5E,
   ADPCH as byte absolute $0F5F,
   ADCON0 as byte absolute $0F60,
   ADPREV as byte absolute $0F61,
   ADPREVL as byte absolute $0F61,
   ADPREVH as byte absolute $0F62,
   ADRES as byte absolute $0F63,
   ADRESL as byte absolute $0F63,
   ADRESH as byte absolute $0F64,
   ADSTAT as byte absolute $0F65,
   ADRPT as byte absolute $0F66,
   ADCNT as byte absolute $0F67,
   ADSTPT as byte absolute $0F68,
   ADSTPTL as byte absolute $0F68,
   ADSTPTH as byte absolute $0F69,
   ADLTH as byte absolute $0F6A,
   ADLTHL as byte absolute $0F6A,
   ADLTHH as byte absolute $0F6B,
   ADUTH as byte absolute $0F6C,
   ADUTHL as byte absolute $0F6C,
   ADUTHH as byte absolute $0F6D,
   ADERR as byte absolute $0F6E,
   ADERRL as byte absolute $0F6E,
   ADERRH as byte absolute $0F6F,
   ADACC as byte absolute $0F70,
   ADACCL as byte absolute $0F70,
   ADACCH as byte absolute $0F71,
   ADFLTR as byte absolute $0F72,
   ADFLTRL as byte absolute $0F72,
   ADFLTRH as byte absolute $0F73,
   NVMADR as byte absolute $0F74,
   NVMADRL as byte absolute $0F74,
   NVMADRH as byte absolute $0F75,
   NVMDAT as byte absolute $0F76,
   NVMCON1 as byte absolute $0F77,
   NVMCON2 as byte absolute $0F78,
   LATA as byte absolute $0F79,
   LATB as byte absolute $0F7A,
   LATC as byte absolute $0F7B,
   LATD as byte absolute $0F7C,
   LATE as byte absolute $0F7D,
   LATF as byte absolute $0F7E,
   LATG as byte absolute $0F7F,
   LATH as byte absolute $0F80,
   DDRA as byte absolute $0F81,
   TRISA as byte absolute $0F81,
   DDRB as byte absolute $0F82,
   TRISB as byte absolute $0F82,
   DDRC as byte absolute $0F83,
   TRISC as byte absolute $0F83,
   DDRD as byte absolute $0F84,
   TRISD as byte absolute $0F84,
   DDRE as byte absolute $0F85,
   TRISE as byte absolute $0F85,
   TRISF as byte absolute $0F86,
   TRISG as byte absolute $0F87,
   TRISH as byte absolute $0F88,
   SSP1BUF as byte absolute $0F91,
   SSPBUF as byte absolute $0F91,
   SSP1ADD as byte absolute $0F92,
   SSPADD as byte absolute $0F92,
   SSP1MSK as byte absolute $0F93,
   SSPMSK as byte absolute $0F93,
   SSP1STAT as byte absolute $0F94,
   SSPSTAT as byte absolute $0F94,
   SSP1CON1 as byte absolute $0F95,
   SSPCON1 as byte absolute $0F95,
   SSP1CON2 as byte absolute $0F96,
   SSPCON2 as byte absolute $0F96,
   SSP1CON3 as byte absolute $0F97,
   SSPCON3 as byte absolute $0F97,
   RC1REG as byte absolute $0F98,
   RCREG as byte absolute $0F98,
   RCREG1 as byte absolute $0F98,
   TX1REG as byte absolute $0F99,
   TXREG as byte absolute $0F99,
   TXREG1 as byte absolute $0F99,
   SP1BRG as byte absolute $0F9A,
   SP1BRGL as byte absolute $0F9A,
   SPBRG as byte absolute $0F9A,
   SPBRG1 as byte absolute $0F9A,
   SPBRGL as byte absolute $0F9A,
   SP1BRGH as byte absolute $0F9B,
   SPBRGH as byte absolute $0F9B,
   SPBRGH1 as byte absolute $0F9B,
   RC1STA as byte absolute $0F9C,
   RCSTA as byte absolute $0F9C,
   RCSTA1 as byte absolute $0F9C,
   TX1STA as byte absolute $0F9D,
   TXSTA as byte absolute $0F9D,
   TXSTA1 as byte absolute $0F9D,
   BAUD1CON as byte absolute $0F9E,
   BAUDCON as byte absolute $0F9E,
   BAUDCON1 as byte absolute $0F9E,
   BAUDCTL as byte absolute $0F9E,
   BAUDCTL1 as byte absolute $0F9E,
   PWM7DCL as byte absolute $0F9F,
   PWM7DCH as byte absolute $0FA0,
   PWM7CON as byte absolute $0FA1,
   PWM6DCL as byte absolute $0FA2,
   PWM6DCH as byte absolute $0FA3,
   PWM6CON as byte absolute $0FA4,
   CCPR2 as byte absolute $0FA5,
   CCPR2L as byte absolute $0FA5,
   CCPR2H as byte absolute $0FA6,
   CCP2CON as byte absolute $0FA7,
   CCP2CAP as byte absolute $0FA8,
   CCPR1 as byte absolute $0FA9,
   CCPR1L as byte absolute $0FA9,
   CCPR1H as byte absolute $0FAA,
   CCP1CON as byte absolute $0FAB,
   CCP1CAP as byte absolute $0FAC,
   CCPTMRS0 as byte absolute $0FAD,
   CCPTMRS1 as byte absolute $0FAE,
   T6TMR as byte absolute $0FAF,
   TMR6 as byte absolute $0FAF,
   PR6 as byte absolute $0FB0,
   T6PR as byte absolute $0FB0,
   T6CON as byte absolute $0FB1,
   T6HLT as byte absolute $0FB2,
   T6CLKCON as byte absolute $0FB3,
   T6RST as byte absolute $0FB4,
   T4TMR as byte absolute $0FB5,
   TMR4 as byte absolute $0FB5,
   PR4 as byte absolute $0FB6,
   T4PR as byte absolute $0FB6,
   T4CON as byte absolute $0FB7,
   T4HLT as byte absolute $0FB8,
   T4CLKCON as byte absolute $0FB9,
   T4RST as byte absolute $0FBA,
   T2TMR as byte absolute $0FBB,
   TMR2 as byte absolute $0FBB,
   PR2 as byte absolute $0FBC,
   T2PR as byte absolute $0FBC,
   T2CON as byte absolute $0FBD,
   T2HLT as byte absolute $0FBE,
   T2CLKCON as byte absolute $0FBF,
   T2RST as byte absolute $0FC0,
   TMR5 as byte absolute $0FC1,
   TMR5L as byte absolute $0FC1,
   TMR5H as byte absolute $0FC2,
   T5CON as byte absolute $0FC3,
   PR5 as byte absolute $0FC4,
   T5GCON as byte absolute $0FC4,
   T5GATE as byte absolute $0FC5,
   TMR5GATE as byte absolute $0FC5,
   T5CLK as byte absolute $0FC6,
   TMR5CLK as byte absolute $0FC6,
   TMR3 as byte absolute $0FC7,
   TMR3L as byte absolute $0FC7,
   TMR3H as byte absolute $0FC8,
   T3CON as byte absolute $0FC9,
   PR3 as byte absolute $0FCA,
   T3GCON as byte absolute $0FCA,
   T3GATE as byte absolute $0FCB,
   TMR3GATE as byte absolute $0FCB,
   T3CLK as byte absolute $0FCC,
   TMR3CLK as byte absolute $0FCC,
   TMR1 as byte absolute $0FCD,
   TMR1L as byte absolute $0FCD,
   TMR1H as byte absolute $0FCE,
   T1CON as byte absolute $0FCF,
   PR1 as byte absolute $0FD0,
   T1GCON as byte absolute $0FD0,
   T1GATE as byte absolute $0FD1,
   TMR1GATE as byte absolute $0FD1,
   T1CLK as byte absolute $0FD2,
   TMR1CLK as byte absolute $0FD2,
   TMR0 as byte absolute $0FD3,
   TMR0L as byte absolute $0FD3,
   PR0 as byte absolute $0FD4,
   TMR0H as byte absolute $0FD4,
   T0CON0 as byte absolute $0FD5,
   T0CON1 as byte absolute $0FD6,
   PCON0 as byte absolute $0FD7,
   STATUS as byte absolute $0FD8,
   FSR2L as byte absolute $0FD9,
   FSR2H as byte absolute $0FDA,
   PLUSW2 as byte absolute $0FDB,
   PREINC2 as byte absolute $0FDC,
   POSTDEC2 as byte absolute $0FDD,
   POSTINC2 as byte absolute $0FDE,
   INDF2 as byte absolute $0FDF,
   BSR as byte absolute $0FE0,
   FSR1L as byte absolute $0FE1,
   FSR1H as byte absolute $0FE2,
   PLUSW1 as byte absolute $0FE3,
   PREINC1 as byte absolute $0FE4,
   POSTDEC1 as byte absolute $0FE5,
   POSTINC1 as byte absolute $0FE6,
   INDF1 as byte absolute $0FE7,
   WREG as byte absolute $0FE8,
   FSR0L as byte absolute $0FE9,
   FSR0H as byte absolute $0FEA,
   PLUSW0 as byte absolute $0FEB,
   PREINC0 as byte absolute $0FEC,
   POSTDEC0 as byte absolute $0FED,
   POSTINC0 as byte absolute $0FEE,
   INDF0 as byte absolute $0FEF,
   INTCON as byte absolute $0FF2,
   PRODL as byte absolute $0FF3,
   PRODH as byte absolute $0FF4,
   TABLAT as byte absolute $0FF5,
   TBLPTR as byte absolute $0FF6,
   TBLPTRL as byte absolute $0FF6,
   TBLPTRH as byte absolute $0FF7,
   TBLPTRU as byte absolute $0FF8,
   PC as byte absolute $0FF9,
   PCL as byte absolute $0FF9,
   PCLATH as byte absolute $0FFA,
   PCLATU as byte absolute $0FFB,
   STKPTR as byte absolute $0FFC,
   TOS as byte absolute $0FFD,
   TOSL as byte absolute $0FFD,
   TOSH as byte absolute $0FFE,
   TOSU as byte absolute $0FFF

// system ports...
public system port
   PORTA as byte absolute $0F89,
   PORTB as byte absolute $0F8A,
   PORTC as byte absolute $0F8B,
   PORTD as byte absolute $0F8C,
   PORTE as byte absolute $0F8D,
   PORTF as byte absolute $0F8E,
   PORTG as byte absolute $0F8F,
   PORTH as byte absolute $0F90

// alias...
public dim
   PROD as PRODL.AsWord,
   FSR0 as FSR0L.AsWord,
   FSR1 as FSR1L.AsWord,
   FSR2 as FSR2L.AsWord,
   TABLEPTR as TBLPTRL.AsWord

// configuration fuses...
public config
   FEXTOSC(FEXTOSC) = [LP, XT, HS, OFF, ECL, ECM, ECH],
   RSTOSC(RSTOSC) = [HFINTOSC_64MHZ, RESERVED_1, EXTOSC_4PLL, SOSC, LFINTOSC, HFINTOSC_1MHZ, EXTOSC],
   CLKOUTEN(CLKOUTEN) = [ON, OFF],
   CSWEN(CSWEN) = [OFF, ON],
   FCMEN(FCMEN) = [OFF, ON],
   MCLRE(MCLRE) = [INTMCLR, EXTMCLR],
   PWRTE(PWRTE) = [ON, OFF],
   LPBOREN(LPBOREN) = [ON, OFF],
   BOREN(BOREN) = [OFF, ON, NOSLP, SBORDIS],
   BORV(BORV) = [VBOR_285, VBOR_270, VBOR_245, VBOR_2P45],
   ZCD(ZCD) = [ON, OFF],
   PPS1WAY(PPS1WAY) = [OFF, ON],
   STVREN(STVREN) = [OFF, ON],
   DEBUG(DEBUG) = [ON, OFF],
   XINST(XINST) = [ON, OFF],
   WDTCPS(WDTCPS) = [WDTCPS_0, WDTCPS_1, WDTCPS_2, WDTCPS_3, WDTCPS_4, WDTCPS_5, WDTCPS_6, WDTCPS_7, WDTCPS_8, WDTCPS_9, WDTCPS_10, WDTCPS_11, WDTCPS_12, WDTCPS_13, WDTCPS_14, WDTCPS_15, WDTCPS_16, WDTCPS_17, WDTCPS_18, WDTCPS_19, WDTCPS_20, WDTCPS_21, WDTCPS_22, WDTCPS_23, WDTCPS_24, WDTCPS_25, WDTCPS_26, WDTCPS_27, WDTCPS_28, WDTCPS_29, WDTCPS_30, WDTCPS_31],
   WDTE(WDTE) = [OFF, SWDTEN, NSLEEP, ON],
   WDTCWS(WDTCWS) = [WDTCWS_0, WDTCWS_1, WDTCWS_2, WDTCWS_3, WDTCWS_4, WDTCWS_5, WDTCWS_6, WDTCWS_7],
   WDTCCS(WDTCCS) = [LFINTOSC, HFINTOSC, SC],
   WRT0(WRT0) = [ON, OFF],
   WRT1(WRT1) = [ON, OFF],
   WRT2(WRT2) = [ON, OFF],
   WRT3(WRT3) = [ON, OFF],
   WRTC(WRTC) = [ON, OFF],
   WRTB(WRTB) = [ON, OFF],
   WRTD(WRTD) = [ON, OFF],
   SCANE(SCANE) = [OFF, ON],
   LVP(LVP) = [OFF, ON],
   CP(CP) = [ON, OFF],
   CPD(CPD) = [ON, OFF],
   EBTR0(EBTR0) = [ON, OFF],
   EBTR1(EBTR1) = [ON, OFF],
   EBTR2(EBTR2) = [ON, OFF],
   EBTR3(EBTR3) = [ON, OFF],
   EBTRB(EBTRB) = [ON, OFF]

// default fuses...
config
   FEXTOSC = HS,
   RSTOSC = HFINTOSC_1MHZ,
   FCMEN = OFF,
   BOREN = ON,
   STVREN = ON,
   DEBUG = OFF,
   XINST = OFF,
   WDTE = OFF,
   LVP = OFF

//
// SF interrupt startup code fix for IPEN (K40, Q10, and others)
//
// the compiler assumes that the IPEN interrupt priority bit is in RCON.bits(7)
// some devices have moved this bit to INTCON.bits(5), and have no RCON register
// to satisfy the compiler we define an RCON register symbol at the same
// address as the PRODL register... that way code will compile and it does
// no harm when the compiler sets/clears the bit.
// when using interrupts SF will set/clear RCON.bits(7) automatically at the
// start of MAIN, so right after that we check the bit (which is now PRODL.7)
// and set the real IPEN bit in INTCON to match.
// this code will execute even if not using interrupts, but should have no
// detrimental effects
//
#if (_ipen = 1)
// the real IPEN bit
public dim IPEN as INTCON.bits(5)
// define dummy RCON as PRODL to satisfy compiler
private system RCON as byte absolute $0FF3
asm
  RCON EQU 0x0FF3
end asm

// replicate the RCON.bits(7) setting into the real IPEN
IPEN = 0
if (RCON.bits(7) = 1) then
    IPEN = 1
endif
#endif      // (_ipen = 1)

// comment out #define to skip fix for K40 A3 errata
#define _K40_A3_ERRATA
#if defined(_K40_A3_ERRATA)
// K40 family silicon rev A3 errata:
//   TBLRD requires NVMREG value to point to appropriate memory
// The affected silicon revisions of the PIC18FXXK40 devices improperly require the
// NVMREG<1:0> bits in the NVMCON register to be set for TBLRD access of the various
// memory regions. This sets NVMREG to point to flash code memory.
NVMCON1.bits(7) = 1
NVMCON1.bits(6) = 0
#endif    // _K40_A3_ERRATA

end module
