{
****************************************************************
*  Name    : 18F8493                                           *
*  Author  : David John Barker                                 *
*  Notice  : Copyright (c) 2018 Mecanique                      *
*          : All Rights Reserved                               *
*  Date    : 7/29/2018                                         *
****************************************************************
}

module SystemTypes

// system header...
#const _core = $0012                   // processor core
#const _ram_banks = $03                // 3 RAM bank(s) used
#variable _maxaccess = $60             // access ram is 96 bytes
#variable _maxram = $0300              // 768 bytes of user RAM
#variable _maxrom = $004000            // 16 KB of program ROM
#const _eeprom = $0000                 // 0 bytes of EEPROM
#const _ports = $09                    // 9 available ports
#const _no_porti = $01                 // no PORTI
#const _pps = $00                      // PPS module NOT supported
#const _ccp = $02                      // 2 CCP module(s) available
#const _eccp = $00                     // ECCP module NOT supported
#const _mssp = $01                     // 1 MSSP module(s) available
#const _usart = $02                    // 2 USART(s) available
#const _adc = $0C                      // 12 ADC channels available
#const _adres = $0A                    // 10 bit ADC resolution
#const _comparator = $02               // 2 comparator(s) available
#const _psp = $00                      // Parallel Slave Port (PSP) is NOT supported
#const _can = $00                      // onboard CAN is NOT supported
#const _usb = $00                      // USB is NOT supported
#const _ethernet = $00                 // onboard Ethernet is NOT supported
#const _flash_write = $01              // FLASH has write capability
#const _wdt_type = $00                 // WDT watchdog
#const _ipen = $00                     // IPEN located in RCON

// special function registers...
public system
   LCDPS as byte absolute $0F58,
   LCDCON as byte absolute $0F59,
   LCDSE0 as byte absolute $0F5A,
   LCDSE1 as byte absolute $0F5B,
   LCDSE2 as byte absolute $0F5C,
   LCDSE3 as byte absolute $0F5D,
   LCDSE4 as byte absolute $0F5E,
   LCDSE5 as byte absolute $0F5F,
   LCDDATA0 as byte absolute $0F60,
   LCDDATA1 as byte absolute $0F61,
   LCDDATA2 as byte absolute $0F62,
   LCDDATA3 as byte absolute $0F63,
   LCDDATA4 as byte absolute $0F64,
   LCDDATA5 as byte absolute $0F65,
   LCDDATA6 as byte absolute $0F66,
   LCDDATA7 as byte absolute $0F67,
   LCDDATA8 as byte absolute $0F68,
   LCDDATA9 as byte absolute $0F69,
   LCDDATA10 as byte absolute $0F6A,
   RCSTA2 as byte absolute $0F6B,
   TXSTA2 as byte absolute $0F6C,
   TXREG2 as byte absolute $0F6D,
   RCREG2 as byte absolute $0F6E,
   SPBRG2 as byte absolute $0F6F,
   LCDDATA11 as byte absolute $0F70,
   LCDDATA12 as byte absolute $0F71,
   LCDDATA13 as byte absolute $0F72,
   LCDDATA14 as byte absolute $0F73,
   LCDDATA15 as byte absolute $0F74,
   LCDDATA16 as byte absolute $0F75,
   LCDDATA17 as byte absolute $0F76,
   LCDDATA18 as byte absolute $0F77,
   LCDDATA19 as byte absolute $0F78,
   LCDDATA20 as byte absolute $0F79,
   LCDDATA21 as byte absolute $0F7A,
   LCDDATA22 as byte absolute $0F7B,
   LCDDATA23 as byte absolute $0F7C,
   BAUDCON1 as byte absolute $0F7E,
   BAUDCTL1 as byte absolute $0F7E,
   SPBRGH1 as byte absolute $0F7F,
   LATA as byte absolute $0F89,
   LATB as byte absolute $0F8A,
   LATC as byte absolute $0F8B,
   LATD as byte absolute $0F8C,
   LATE as byte absolute $0F8D,
   LATF as byte absolute $0F8E,
   LATG as byte absolute $0F8F,
   LATH as byte absolute $0F90,
   LATJ as byte absolute $0F91,
   DDRA as byte absolute $0F92,
   TRISA as byte absolute $0F92,
   DDRB as byte absolute $0F93,
   TRISB as byte absolute $0F93,
   DDRC as byte absolute $0F94,
   TRISC as byte absolute $0F94,
   DDRD as byte absolute $0F95,
   TRISD as byte absolute $0F95,
   DDRE as byte absolute $0F96,
   TRISE as byte absolute $0F96,
   DDRF as byte absolute $0F97,
   TRISF as byte absolute $0F97,
   DDRG as byte absolute $0F98,
   TRISG as byte absolute $0F98,
   DDRH as byte absolute $0F99,
   TRISH as byte absolute $0F99,
   DDRJ as byte absolute $0F9A,
   TRISJ as byte absolute $0F9A,
   OSCTUNE as byte absolute $0F9B,
   MEMCON as byte absolute $0F9C,
   PIE1 as byte absolute $0F9D,
   PIR1 as byte absolute $0F9E,
   IPR1 as byte absolute $0F9F,
   PIE2 as byte absolute $0FA0,
   PIR2 as byte absolute $0FA1,
   IPR2 as byte absolute $0FA2,
   PIE3 as byte absolute $0FA3,
   PIR3 as byte absolute $0FA4,
   IPR3 as byte absolute $0FA5,
   RCSTA as byte absolute $0FAB,
   RCSTA1 as byte absolute $0FAB,
   TXSTA as byte absolute $0FAC,
   TXSTA1 as byte absolute $0FAC,
   TXREG as byte absolute $0FAD,
   TXREG1 as byte absolute $0FAD,
   RCREG as byte absolute $0FAE,
   RCREG1 as byte absolute $0FAE,
   SPBRG as byte absolute $0FAF,
   SPBRG1 as byte absolute $0FAF,
   T3CON as byte absolute $0FB1,
   TMR3 as byte absolute $0FB2,
   TMR3L as byte absolute $0FB2,
   TMR3H as byte absolute $0FB3,
   CMCON as byte absolute $0FB4,
   CVRCON as byte absolute $0FB5,
   CCP2CON as byte absolute $0FBA,
   CCPR2 as byte absolute $0FBB,
   CCPR2L as byte absolute $0FBB,
   CCPR2H as byte absolute $0FBC,
   CCP1CON as byte absolute $0FBD,
   CCPR1 as byte absolute $0FBE,
   CCPR1L as byte absolute $0FBE,
   CCPR1H as byte absolute $0FBF,
   ADCON2 as byte absolute $0FC0,
   ADCON1 as byte absolute $0FC1,
   ADCON0 as byte absolute $0FC2,
   ADRES as byte absolute $0FC3,
   ADRESL as byte absolute $0FC3,
   ADRESH as byte absolute $0FC4,
   SSPCON2 as byte absolute $0FC5,
   SSPCON1 as byte absolute $0FC6,
   SSPSTAT as byte absolute $0FC7,
   SSPADD as byte absolute $0FC8,
   SSPBUF as byte absolute $0FC9,
   T2CON as byte absolute $0FCA,
   PR2 as byte absolute $0FCB,
   TMR2 as byte absolute $0FCC,
   T1CON as byte absolute $0FCD,
   TMR1 as byte absolute $0FCE,
   TMR1L as byte absolute $0FCE,
   TMR1H as byte absolute $0FCF,
   RCON as byte absolute $0FD0,
   WDTCON as byte absolute $0FD1,
   HLVDCON as byte absolute $0FD2,
   OSCCON as byte absolute $0FD3,
   T0CON as byte absolute $0FD5,
   TMR0 as byte absolute $0FD6,
   TMR0L as byte absolute $0FD6,
   TMR0H as byte absolute $0FD7,
   STATUS as byte absolute $0FD8,
   FSR2L as byte absolute $0FD9,
   FSR2H as byte absolute $0FDA,
   PLUSW2 as byte absolute $0FDB,
   PREINC2 as byte absolute $0FDC,
   POSTDEC2 as byte absolute $0FDD,
   POSTINC2 as byte absolute $0FDE,
   INDF2 as byte absolute $0FDF,
   BSR as byte absolute $0FE0,
   FSR1L as byte absolute $0FE1,
   FSR1H as byte absolute $0FE2,
   PLUSW1 as byte absolute $0FE3,
   PREINC1 as byte absolute $0FE4,
   POSTDEC1 as byte absolute $0FE5,
   POSTINC1 as byte absolute $0FE6,
   INDF1 as byte absolute $0FE7,
   WREG as byte absolute $0FE8,
   FSR0L as byte absolute $0FE9,
   FSR0H as byte absolute $0FEA,
   PLUSW0 as byte absolute $0FEB,
   PREINC0 as byte absolute $0FEC,
   POSTDEC0 as byte absolute $0FED,
   POSTINC0 as byte absolute $0FEE,
   INDF0 as byte absolute $0FEF,
   INTCON3 as byte absolute $0FF0,
   INTCON2 as byte absolute $0FF1,
   INTCON as byte absolute $0FF2,
   PRODL as byte absolute $0FF3,
   PRODH as byte absolute $0FF4,
   TABLAT as byte absolute $0FF5,
   TBLPTR as byte absolute $0FF6,
   TBLPTRL as byte absolute $0FF6,
   TBLPTRH as byte absolute $0FF7,
   TBLPTRU as byte absolute $0FF8,
   PC as byte absolute $0FF9,
   PCL as byte absolute $0FF9,
   PCLATH as byte absolute $0FFA,
   PCLATU as byte absolute $0FFB,
   STKPTR as byte absolute $0FFC,
   TOS as byte absolute $0FFD,
   TOSL as byte absolute $0FFD,
   TOSH as byte absolute $0FFE,
   TOSU as byte absolute $0FFF

// system ports...
public system port
   PORTA as byte absolute $0F80,
   PORTB as byte absolute $0F81,
   PORTC as byte absolute $0F82,
   PORTD as byte absolute $0F83,
   PORTE as byte absolute $0F84,
   PORTF as byte absolute $0F85,
   PORTG as byte absolute $0F86,
   PORTH as byte absolute $0F87,
   PORTJ as byte absolute $0F88

// alias...
public dim
   PROD as PRODL.AsWord,
   FSR0 as FSR0L.AsWord,
   FSR1 as FSR1L.AsWord,
   FSR2 as FSR2L.AsWord,
   TABLEPTR as TBLPTRL.AsWord

// configuration fuses...
public config
   OSC(OSC) = [LP, XT, HS, RC, EC, ECIO, HSPLL, RCIO, INTIO67, INTIO7],
   FCMEN(FCMEN) = [OFF, ON],
   IESO(IESO) = [OFF, ON],
   PWRT(PWRT) = [ON, OFF],
   BOREN(BOREN) = [OFF, ON, NOSLP, SBORDIS],
   BORV(BORV) = [0, 1, 2, 3],
   WDT(WDT) = [OFF, ON],
   WDTPS(WDTPS) = [1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768],
   CCP2MX(CCP2MX) = [PORTBE, PORTC],
   LPT1OSC(LPT1OSC) = [OFF, ON],
   MCLRE(MCLRE) = [OFF, ON],
   STVREN(STVREN) = [OFF, ON],
   XINST(XINST) = [OFF, ON],
   DEBUG(DEBUG) = [ON, OFF],
   CP(CP) = [ON, OFF]

// default fuses...
config
   OSC = HS,
   FCMEN = OFF,
   IESO = OFF,
   PWRT = ON,
   BOREN = ON,
   WDT = OFF,
   WDTPS = 128,
   STVREN = ON,
   XINST = OFF,
   DEBUG = OFF

end module
