{
****************************************************************
*  Name    : 18F86K90                                          *
*  Author  : David John Barker                                 *
*  Notice  : Copyright (c) 2018 Mecanique                      *
*          : All Rights Reserved                               *
*  Date    : 7/29/2018                                         *
****************************************************************
}

module SystemTypes

// system header...
#const _core = $0012                   // processor core
#const _ram_banks = $0F                // 15 RAM bank(s) used
#variable _maxaccess = $60             // access ram is 96 bytes
#variable _maxram = $0EF4              // 3828 bytes of user RAM
#variable _maxrom = $010000            // 64 KB of program ROM
#const _eeprom = $0400                 // 1024 bytes of EEPROM
#const _eeprom_start = $F00000         // EEPROM start address
#const _nvmcon = $00                   // eeprom uses EECON1
#const _ports = $09                    // 9 available ports
#const _no_porti = $01                 // no PORTI
#const _pps = $00                      // PPS module NOT supported
#const _ccp = $06                      // 6 CCP module(s) available
#const _eccp = $03                     // 3 ECCP module(s) available
#const _mssp = $02                     // 2 MSSP module(s) available
#const _usart = $02                    // 2 USART(s) available
#const _adc = $15                      // 21 ADC channels available
#const _adres = $0A                    // 10 bit ADC resolution
#const _comparator = $03               // 3 comparator(s) available
#const _psp = $00                      // Parallel Slave Port (PSP) is NOT supported
#const _can = $00                      // onboard CAN is NOT supported
#const _usb = $00                      // USB is NOT supported
#const _ethernet = $00                 // onboard Ethernet is NOT supported
#const _flash_write = $01              // FLASH has write capability
#const _wdt_type = $03                 // WDTEN watchdog (w/SWDTDIS)
#const _ipen = $00                     // IPEN located in RCON

// special function registers...
public system
   LCDCON as byte absolute $0EF4,
   LCDPS as byte absolute $0EF5,
   LCDSE0 as byte absolute $0EF6,
   LCDSE1 as byte absolute $0EF7,
   LCDSE2 as byte absolute $0EF8,
   LCDSE3 as byte absolute $0EF9,
   LCDSE4 as byte absolute $0EFA,
   LCDSE5 as byte absolute $0EFB,
   LCDRL as byte absolute $0EFC,
   LCDREF as byte absolute $0EFD,
   SSP2CON2 as byte absolute $0EFE,
   SSP2CON1 as byte absolute $0EFF,
   SSP2STAT as byte absolute $0F00,
   SSP2ADD as byte absolute $0F01,
   SSP2BUF as byte absolute $0F02,
   T4CON as byte absolute $0F03,
   PR4 as byte absolute $0F04,
   TMR4 as byte absolute $0F05,
   CCP7CON as byte absolute $0F06,
   CCPR7 as byte absolute $0F07,
   CCPR7L as byte absolute $0F07,
   CCPR7H as byte absolute $0F08,
   CCP6CON as byte absolute $0F09,
   CCPR6 as byte absolute $0F0A,
   CCPR6L as byte absolute $0F0A,
   CCPR6H as byte absolute $0F0B,
   CCP5CON as byte absolute $0F0C,
   CCPR5 as byte absolute $0F0D,
   CCPR5L as byte absolute $0F0D,
   CCPR5H as byte absolute $0F0E,
   CCP4CON as byte absolute $0F0F,
   CCPR4 as byte absolute $0F10,
   CCPR4L as byte absolute $0F10,
   CCPR4H as byte absolute $0F11,
   T5GCON as byte absolute $0F12,
   T5CON as byte absolute $0F13,
   TMR5 as byte absolute $0F14,
   TMR5L as byte absolute $0F14,
   TMR5H as byte absolute $0F15,
   PMD3 as byte absolute $0F16,
   PMD2 as byte absolute $0F17,
   PMD1 as byte absolute $0F18,
   PMD0 as byte absolute $0F19,
   PSTR3CON as byte absolute $0F1A,
   PSTR2CON as byte absolute $0F1B,
   TXREG2 as byte absolute $0F1C,
   RCREG2 as byte absolute $0F1D,
   SPBRG2 as byte absolute $0F1E,
   SPBRGH2 as byte absolute $0F1F,
   BAUDCON2 as byte absolute $0F20,
   TXSTA2 as byte absolute $0F21,
   RCSTA2 as byte absolute $0F22,
   ANCON2 as byte absolute $0F23,
   ANCON1 as byte absolute $0F24,
   ANCON0 as byte absolute $0F25,
   ODCON3 as byte absolute $0F27,
   ODCON2 as byte absolute $0F28,
   ODCON1 as byte absolute $0F29,
   REFOCON as byte absolute $0F2A,
   CCPTMRS2 as byte absolute $0F2B,
   CCPTMRS1 as byte absolute $0F2C,
   CCPTMRS0 as byte absolute $0F2D,
   CM3CON as byte absolute $0F2E,
   CM3CON1 as byte absolute $0F2E,
   CM2CON as byte absolute $0F2F,
   CM2CON1 as byte absolute $0F2F,
   T12CON as byte absolute $0F30,
   PR12 as byte absolute $0F31,
   TMR12 as byte absolute $0F32,
   T10CON as byte absolute $0F33,
   PR10 as byte absolute $0F34,
   TMR10 as byte absolute $0F35,
   T8CON as byte absolute $0F36,
   PR8 as byte absolute $0F37,
   TMR8 as byte absolute $0F38,
   T6CON as byte absolute $0F39,
   PR6 as byte absolute $0F3A,
   TMR6 as byte absolute $0F3B,
   T7GCON as byte absolute $0F3C,
   T7CON as byte absolute $0F3D,
   TMR7 as byte absolute $0F3E,
   TMR7L as byte absolute $0F3E,
   TMR7H as byte absolute $0F3F,
   CCP10CON as byte absolute $0F40,
   CCPR10 as byte absolute $0F41,
   CCPR10L as byte absolute $0F41,
   CCPR10H as byte absolute $0F42,
   CCP9CON as byte absolute $0F43,
   CCPR9 as byte absolute $0F44,
   CCPR9L as byte absolute $0F44,
   CCPR9H as byte absolute $0F45,
   CCP8CON as byte absolute $0F46,
   CCPR8 as byte absolute $0F47,
   CCPR8L as byte absolute $0F47,
   CCPR8H as byte absolute $0F48,
   CCP3CON as byte absolute $0F49,
   CCPR3 as byte absolute $0F4A,
   CCPR3L as byte absolute $0F4A,
   CCPR3H as byte absolute $0F4B,
   ECCP3DEL as byte absolute $0F4C,
   PWM3CON as byte absolute $0F4C,
   ECCP3AS as byte absolute $0F4D,
   CCP2CON as byte absolute $0F4E,
   ECCP2CON as byte absolute $0F4E,
   CCPR2 as byte absolute $0F4F,
   CCPR2L as byte absolute $0F4F,
   CCPR2H as byte absolute $0F50,
   ECCP2DEL as byte absolute $0F51,
   PWM2CON as byte absolute $0F51,
   ECCP2AS as byte absolute $0F52,
   PADCFG1 as byte absolute $0F53,
   CM1CON as byte absolute $0F54,
   CM1CON1 as byte absolute $0F54,
   CTMUICON as byte absolute $0F55,
   CTMUCONL as byte absolute $0F56,
   CTMUCONH as byte absolute $0F57,
   ALRMVAL as byte absolute $0F58,
   ALRMVALL as byte absolute $0F58,
   ALRMVALH as byte absolute $0F59,
   ALRMRPT as byte absolute $0F5A,
   ALRMCFG as byte absolute $0F5B,
   RTCVAL as byte absolute $0F5C,
   RTCVALL as byte absolute $0F5C,
   RTCVALH as byte absolute $0F5D,
   RTCCAL as byte absolute $0F5E,
   RTCCFG as byte absolute $0F5F,
   PIE6 as byte absolute $0F60,
   EEDATA as byte absolute $0F61,
   EEADR as byte absolute $0F62,
   EEADRH as byte absolute $0F63,
   OSCCON2 as byte absolute $0F64,
   BAUDCON as byte absolute $0F65,
   BAUDCON1 as byte absolute $0F65,
   BAUDCTL as byte absolute $0F65,
   LCDDATA0 as byte absolute $0F66,
   LCDDATA1 as byte absolute $0F67,
   LCDDATA2 as byte absolute $0F68,
   LCDDATA3 as byte absolute $0F69,
   LCDDATA4 as byte absolute $0F6A,
   LCDDATA5 as byte absolute $0F6B,
   LCDDATA6 as byte absolute $0F6C,
   LCDDATA7 as byte absolute $0F6D,
   LCDDATA8 as byte absolute $0F6E,
   LCDDATA9 as byte absolute $0F6F,
   LCDDATA10 as byte absolute $0F70,
   LCDDATA11 as byte absolute $0F71,
   LCDDATA12 as byte absolute $0F72,
   LCDDATA13 as byte absolute $0F73,
   LCDDATA14 as byte absolute $0F74,
   LCDDATA15 as byte absolute $0F75,
   LCDDATA16 as byte absolute $0F76,
   LCDDATA17 as byte absolute $0F77,
   LCDDATA18 as byte absolute $0F78,
   LCDDATA19 as byte absolute $0F79,
   LCDDATA20 as byte absolute $0F7A,
   LCDDATA21 as byte absolute $0F7B,
   LCDDATA22 as byte absolute $0F7C,
   LCDDATA23 as byte absolute $0F7D,
   EECON2 as byte absolute $0F7E,
   EECON1 as byte absolute $0F7F,
   LATA as byte absolute $0F89,
   LATB as byte absolute $0F8A,
   LATC as byte absolute $0F8B,
   LATD as byte absolute $0F8C,
   LATE as byte absolute $0F8D,
   LATF as byte absolute $0F8E,
   LATG as byte absolute $0F8F,
   LATH as byte absolute $0F90,
   LATJ as byte absolute $0F91,
   TRISA as byte absolute $0F92,
   TRISB as byte absolute $0F93,
   TRISC as byte absolute $0F94,
   TRISD as byte absolute $0F95,
   TRISE as byte absolute $0F96,
   TRISF as byte absolute $0F97,
   TRISG as byte absolute $0F98,
   TRISH as byte absolute $0F99,
   TRISJ as byte absolute $0F9A,
   OSCTUNE as byte absolute $0F9B,
   PSTR1CON as byte absolute $0F9C,
   PIE1 as byte absolute $0F9D,
   PIR1 as byte absolute $0F9E,
   IPR1 as byte absolute $0F9F,
   PIE2 as byte absolute $0FA0,
   PIR2 as byte absolute $0FA1,
   IPR2 as byte absolute $0FA2,
   PIE3 as byte absolute $0FA3,
   PIR3 as byte absolute $0FA4,
   IPR3 as byte absolute $0FA5,
   PIR6 as byte absolute $0FA6,
   HLVDCON as byte absolute $0FA8,
   IPR6 as byte absolute $0FA9,
   T1GCON as byte absolute $0FAA,
   RCSTA as byte absolute $0FAB,
   RCSTA1 as byte absolute $0FAB,
   TXSTA as byte absolute $0FAC,
   TXSTA1 as byte absolute $0FAC,
   TXREG as byte absolute $0FAD,
   TXREG1 as byte absolute $0FAD,
   RCREG as byte absolute $0FAE,
   RCREG1 as byte absolute $0FAE,
   SPBRG as byte absolute $0FAF,
   SPBRG1 as byte absolute $0FAF,
   T3GCON as byte absolute $0FB0,
   T3CON as byte absolute $0FB1,
   TMR3 as byte absolute $0FB2,
   TMR3L as byte absolute $0FB2,
   TMR3H as byte absolute $0FB3,
   CMSTAT as byte absolute $0FB4,
   CMSTATUS as byte absolute $0FB4,
   CVRCON as byte absolute $0FB5,
   PIE4 as byte absolute $0FB6,
   PIR4 as byte absolute $0FB7,
   IPR4 as byte absolute $0FB8,
   PIE5 as byte absolute $0FB9,
   PIR5 as byte absolute $0FBA,
   CCP1CON as byte absolute $0FBB,
   ECCP1CON as byte absolute $0FBB,
   CCPR1 as byte absolute $0FBC,
   CCPR1L as byte absolute $0FBC,
   CCPR1H as byte absolute $0FBD,
   ECCP1DEL as byte absolute $0FBE,
   PWM1CON as byte absolute $0FBE,
   ECCP1AS as byte absolute $0FBF,
   ADCON2 as byte absolute $0FC0,
   ADCON1 as byte absolute $0FC1,
   ADCON0 as byte absolute $0FC2,
   ADRES as byte absolute $0FC3,
   ADRESL as byte absolute $0FC3,
   ADRESH as byte absolute $0FC4,
   SSP1CON2 as byte absolute $0FC5,
   SSPCON2 as byte absolute $0FC5,
   SSP1CON1 as byte absolute $0FC6,
   SSPCON1 as byte absolute $0FC6,
   SSP1STAT as byte absolute $0FC7,
   SSPSTAT as byte absolute $0FC7,
   SSP1ADD as byte absolute $0FC8,
   SSPADD as byte absolute $0FC8,
   SSP1BUF as byte absolute $0FC9,
   SSPBUF as byte absolute $0FC9,
   T2CON as byte absolute $0FCA,
   PR2 as byte absolute $0FCB,
   TMR2 as byte absolute $0FCC,
   T1CON as byte absolute $0FCD,
   TMR1 as byte absolute $0FCE,
   TMR1L as byte absolute $0FCE,
   TMR1H as byte absolute $0FCF,
   RCON as byte absolute $0FD0,
   WDTCON as byte absolute $0FD1,
   IPR5 as byte absolute $0FD2,
   OSCCON as byte absolute $0FD3,
   SPBRGH1 as byte absolute $0FD4,
   T0CON as byte absolute $0FD5,
   TMR0 as byte absolute $0FD6,
   TMR0L as byte absolute $0FD6,
   TMR0H as byte absolute $0FD7,
   STATUS as byte absolute $0FD8,
   FSR2L as byte absolute $0FD9,
   FSR2H as byte absolute $0FDA,
   PLUSW2 as byte absolute $0FDB,
   PREINC2 as byte absolute $0FDC,
   POSTDEC2 as byte absolute $0FDD,
   POSTINC2 as byte absolute $0FDE,
   INDF2 as byte absolute $0FDF,
   BSR as byte absolute $0FE0,
   FSR1L as byte absolute $0FE1,
   FSR1H as byte absolute $0FE2,
   PLUSW1 as byte absolute $0FE3,
   PREINC1 as byte absolute $0FE4,
   POSTDEC1 as byte absolute $0FE5,
   POSTINC1 as byte absolute $0FE6,
   INDF1 as byte absolute $0FE7,
   WREG as byte absolute $0FE8,
   FSR0L as byte absolute $0FE9,
   FSR0H as byte absolute $0FEA,
   PLUSW0 as byte absolute $0FEB,
   PREINC0 as byte absolute $0FEC,
   POSTDEC0 as byte absolute $0FED,
   POSTINC0 as byte absolute $0FEE,
   INDF0 as byte absolute $0FEF,
   INTCON3 as byte absolute $0FF0,
   INTCON2 as byte absolute $0FF1,
   INTCON as byte absolute $0FF2,
   PRODL as byte absolute $0FF3,
   PRODH as byte absolute $0FF4,
   TABLAT as byte absolute $0FF5,
   TBLPTR as byte absolute $0FF6,
   TBLPTRL as byte absolute $0FF6,
   TBLPTRH as byte absolute $0FF7,
   TBLPTRU as byte absolute $0FF8,
   PC as byte absolute $0FF9,
   PCL as byte absolute $0FF9,
   PCLATH as byte absolute $0FFA,
   PCLATU as byte absolute $0FFB,
   STKPTR as byte absolute $0FFC,
   TOS as byte absolute $0FFD,
   TOSL as byte absolute $0FFD,
   TOSH as byte absolute $0FFE,
   TOSU as byte absolute $0FFF

// system ports...
public system port
   PORTA as byte absolute $0F80,
   PORTB as byte absolute $0F81,
   PORTC as byte absolute $0F82,
   PORTD as byte absolute $0F83,
   PORTE as byte absolute $0F84,
   PORTF as byte absolute $0F85,
   PORTG as byte absolute $0F86,
   PORTH as byte absolute $0F87,
   PORTJ as byte absolute $0F88

// alias...
public dim
   PROD as PRODL.AsWord,
   FSR0 as FSR0L.AsWord,
   FSR1 as FSR1L.AsWord,
   FSR2 as FSR2L.AsWord,
   TABLEPTR as TBLPTRL.AsWord

// configuration fuses...
public config
   RETEN(RETEN) = [OFF, ON],
   INTOSCSEL(INTOSCSEL) = [LOW, HIGH],
   SOSCSEL(SOSCSEL) = [LOW, DIG, HIGH],
   XINST(XINST) = [OFF, ON],
   FOSC(FOSC) = [LP, XT, HS2, HS1, EC3IO, EC3, RC, RCIO, INTIO2, INTIO1, EC2IO, EC2, EC1IO, EC1],
   PLLCFG(PLLCFG) = [OFF, ON],
   FCMEN(FCMEN) = [OFF, ON],
   IESO(IESO) = [OFF, ON],
   PWRTEN(PWRTEN) = [ON, OFF],
   BOREN(BOREN) = [OFF, ON, NOSLP, SBORDIS],
   BORV(BORV) = [0, 1, 2, 3],
   BORPWR(BORPWR) = [LOW, MEDIUM, HIGH, ZPBORMV],
   WDTEN(WDTEN) = [OFF, NOSLP, ON, SWDTDIS],
   WDTPS(WDTPS) = [1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, 65536, 131072, 262144, 524288, 1048576],
   RTCOSC(RTCOSC) = [INTOSCREF, SOSCREF],
   CCP2MX(CCP2MX) = [PORTBE, PORTC],
   ECCPMX(ECCPMX) = [PORTH, PORTE],
   MSSPMSK(MSSPMSK) = [MSK5, MSK7],
   MCLRE(MCLRE) = [OFF, ON],
   STVREN(STVREN) = [OFF, ON],
   BBSIZ(BBSIZ) = [BB1K, BB2K],
   DEBUG(DEBUG) = [ON, OFF],
   CP0(CP0) = [ON, OFF],
   CP1(CP1) = [ON, OFF],
   CP2(CP2) = [ON, OFF],
   CP3(CP3) = [ON, OFF],
   CPB(CPB) = [ON, OFF],
   CPD(CPD) = [ON, OFF],
   WRT0(WRT0) = [ON, OFF],
   WRT1(WRT1) = [ON, OFF],
   WRT2(WRT2) = [ON, OFF],
   WRT3(WRT3) = [ON, OFF],
   WRTC(WRTC) = [ON, OFF],
   WRTB(WRTB) = [ON, OFF],
   WRTD(WRTD) = [ON, OFF],
   EBRT0(EBRT0) = [ON, OFF],
   EBRT1(EBRT1) = [ON, OFF],
   EBRT2(EBRT2) = [ON, OFF],
   EBRT3(EBRT3) = [ON, OFF],
   EBRTB(EBRTB) = [ON, OFF]

// default fuses...
config
   XINST = OFF,
   FCMEN = OFF,
   IESO = OFF,
   BOREN = ON,
   WDTEN = OFF,
   WDTPS = 128,
   STVREN = ON,
   DEBUG = OFF

end module
